unit fPatCheck;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, VA508AccessibilityManager,
  Vcl.CheckLst, Vcl.ExtCtrls, uUtilities, U508Extensions, Vcl.ComCtrls;

type
  TfrmPatientCheck = class(TForm)
    Panel1: TPanel;
    GridPanel1: TGridPanel;
    pnlUnable: TPanel;
    pnlPend: TPanel;
    lbUnable: TListBox;
    btnOK: TButton;
    btnCancel: TButton;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    lblPend: TLabel;
    lblUnable: TLabel;
    lbPend: TListBox;
    va508Footer: TVA508StaticText;
    pnlCancel: TPanel;
    lblCancel: TLabel;
    lbCancel: TListBox;
    lvUnable: TListView;
    lvPend: TListView;
    lvCancel: TListView;
    btnIgnore: TButton;
  private
    { Private declarations }
    procedure SetColumnWidth();
  public
    { Public declarations }
  end;

function DisplayPatientCheck(PatList: tPatRecArry; ViewType: tViewType; ShowIgnore: Boolean): TModalResult;

implementation

uses
  Winapi.CommCtrl, VAUtils;

const
 TxtUnable = 'The following patient(s) have orders that are already pending for the selected provider. Unable to move these orders.';
 TxtCancel = 'The following patient(s) have orders that are pending a transfer from the selected provider. Continuing to transfer these will cancel the current pending transfer.';
 TxtPend = 'The following patient(s) have orders that are in a pending transfer state. This action will overwrite these entries and move these order notifications to the selected provider.';

 TxtUnableOrders = 'The following order(s) have orders that are already pending for the selected provider. Unable to move these orders.';
 TxtPendOrders = 'The following order(s) are in a pending transfer state. This action will overwrite these entries and move these order notifications to the selected provider.';
 TxtCancelOrders = 'The following order(s) are pending a transfer from the selected provider. Continuing to transfer these will cancel the current pending transfer.';

{$R *.dfm}

function DisplayPatientCheck(PatList: tPatRecArry; ViewType: tViewType; ShowIgnore: Boolean): TModalResult;
const
  U = '^';
var
  f: TfrmPatientCheck;
  I, RowsVisible: Integer;
const
  OvrTxt = 'Click OK to overwrite these orders.';

  procedure AddOrderToList(aLv: TListView; aStr: String);
  begin
    //Add New Columns
    if (ViewType = tvtOrder) and (aLv.Columns.Count = 1) then
    begin
      with aLv.Columns.Add do
        Caption := 'Order IEN';
      if alv = f.lvCancel then
      begin
        with aLv.Columns.Add do
          Caption := 'Transfer From';
      end else begin
        with aLv.Columns.Add do
          Caption := 'Transfer To';
      end;
      with aLv.Columns.Add do
        Caption := 'Transfer Effective Date/Time';
    end;

    with aLv.Items.Add do
    begin
      caption := Piece(aStr, U, 1);
      if ViewType = tvtOrder then
      begin
        SubItems.Add(Piece(aStr, U, 2));
        SubItems.Add(Piece(aStr, U, 3));
        SubItems.Add(Piece(aStr, U, 4));
      end;
    end;

  end;

begin
  f := TfrmPatientCheck.Create(Application);
  try
    if ViewType <> tvtOrder then
    begin
      f.lblUnable.Caption := TxtUnable;
      f.lblCancel.Caption := TxtCancel;
      f.lblPend.Caption := TxtPend;
      f.lvUnable.Hint := TxtUnable;
      f.lvCancel.Hint := TxtCancel;
      f.lvPend.Hint := TxtPend;
    end else begin
      f.lblUnable.Caption := TxtUnableOrders;
      f.lblCancel.Caption := TxtCancelOrders;
      f.lblPend.Caption := TxtPendOrders;
      f.lvUnable.Hint := TxtUnableOrders;
      f.lvCancel.Hint := TxtCancelOrders;
      f.lvPend.Hint := TxtPendOrders;
      f.va508Footer.Visible := false;
    end;

    if length(PatList) > 0 then
    begin
      for I := low(PatList) to high(PatList) do
      begin
        if PatList[I].Reason = txrSame then
        begin
           AddOrderToList(f.lvUnable, PatList[I].name);
          //f.lvUnable.AddItem(PatList[I].name, nil);
          if PatList[I].SelObj is TTreeNode then
            TTreeNode(PatList[I].SelObj).Selected := false
          else if PatList[I].SelObj is TListItem then
            TListItem(PatList[I].SelObj).Selected := false;
        end
        else if PatList[I].Reason = txrPend then
           AddOrderToList(f.lvPend, PatList[I].name)
           //f.lvPend.AddItem(PatList[I].name, nil)
        else if PatList[I].Reason = txrCancel then
          AddOrderToList(f.lvCancel, PatList[I].name);
          //f.lvCancel.AddItem(PatList[I].name, nil);
      end;
    end;

    if (f.lvUnable.Items.Count = 0) and (f.lvPend.Items.Count = 0) and (f.lvCancel.Items.Count = 0) then
    begin
      result := mrOk;
      exit;
    end else begin
      RowsVisible := 0;

      if f.lvUnable.Items.Count > 0 then
        Inc(RowsVisible);
      if f.lvPend.Items.Count > 0 then
        Inc(RowsVisible);
      if f.lvCancel.Items.Count > 0 then
        Inc(RowsVisible);

      //Update the size
      if f.lvUnable.Items.Count = 0 then
      begin
        f.pnlUnable.Visible := false;
        f.GridPanel1.RowCollection[0].Value := 0;
      end else
        f.GridPanel1.RowCollection[0].Value := 100/RowsVisible;

      if f.lvPend.Items.Count = 0 then
      begin
       f.pnlPend.Visible := false;
       f.GridPanel1.RowCollection[1].Value := 0;
      end else
        f.GridPanel1.RowCollection[1].Value := 100/RowsVisible;

      if f.lvCancel.Items.Count = 0 then
      begin
        f.pnlCancel.Visible := false;
        f.GridPanel1.RowCollection[2].Value := 0;
      end else
        f.GridPanel1.RowCollection[2].Value := 100/RowsVisible;

      if f.lvPend.Items.Count > 0 then
        f.va508Footer.Caption := f.va508Footer.Caption + ' ' + OvrTxt;


      f.SetColumnWidth;
      f.btnCancel.Visible := (f.lvPend.Items.Count > 0) or (f.lvCancel.Items.Count > 0);
      f.btnIgnore.Visible := f.btnCancel.Visible and ShowIgnore;
      result := f.ShowModal;
    end;
  finally
    f.Release;
  end;
end;


procedure TfrmPatientCheck.SetColumnWidth();
// ******************************************************************************
// Resize the virtual list views based on the new data
// ******************************************************************************

  procedure AutoResizeColumn(const AListView: TListView);
  // ******************************************************************************
  // Auto resize a column
  // ******************************************************************************
  var
    S: string;
    I, AColumn: Integer;
    MaxWidth: Integer;
    ItemWidth: Integer;
  begin
    for AColumn := 0 to AListView.Columns.Count - 1 do
    begin
    // Grab the widht of the columns caption
    MaxWidth := ListView_GetStringWidth(AListView.Handle,
      PChar(AListView.Columns.Items[AColumn].Caption));
    // Grab the width of the text that should display in that column
    for I := 0 to AListView.Items.Count - 1 do
    begin
      // What text are we looking for?
      if AColumn = 0 then
        s := AListView.Items[i].Caption
      else
      begin
        if AColumn - 1 > AListView.Items[i].SubItems.Count then
          continue;
        s :=  AListView.Items[i].SubItems[AColumn - 1];
      end;
      // Get the text width
      ItemWidth := ListView_GetStringWidth(AListView.Handle, PChar(S));
      // Update our width if needed
      if MaxWidth < ItemWidth then
        MaxWidth := ItemWidth;
    end;

    // Pad the column
    MaxWidth := MaxWidth + 14;
    // Set the column width
    AListView.Columns.Items[AColumn].Width := MaxWidth;
  end;
  end;

begin
  if pnlUnable.Visible then
    AutoResizeColumn(lvUnable);
  if pnlPend.Visible then
    AutoResizeColumn(lvPend);
  if pnlCancel.Visible then
    AutoResizeColumn(lvCancel);

end;

end.
