 // Mandatory unit header per GUI SAC, Section 6.1.1
// Package: Provider Role Tool GUI application
// Date Created: August 2017
// Site Name: CPRS EP1 Remote Development Team
// Developers: Scott Stephenson (Delphi), Chris Bell (Delphi)
// Robert Davis (M), Jamie Crumley (M)
// Description: Main form for the Provider Role Tool application
// Note: This application requires OR_3_453 in order to run.

unit fReassignOrders;
// *******************************************************************
// To-Do section
// *******************************************************************
{ TODO -cGeneral -oChris Bell : Code cleanup and comments }
{ TODO -cGeneral -oChris Bell : Optimization }

{ DONE -cGeneral -oChris Bell : test moving items with in transfer and get overide }
{ DONE -oChris Bell -c508 : Testing }
{ DONE -oChris Bell -c508 : Add header manager }
{ DONE -oChris Bell -c508 : add way to filter audit }
{ DONE -oChris Bell : fix the date time }
{ DONE : limit destination provider not the same source provider }
{ DONE -oChris Bell : Update to pass in override or cancel flag }
// *******************************************************************
// About
// *******************************************************************
// This is a GUI prototype supporting the reassignment of patients from one provider to another
// The nominal program flow follows...
// - user selects current/former provider and date range via lookup widget
// - program displays list of patients with outstanding orders at left
// - user selects one or more new providers and adds to tree view at right
// - user drags and drops patients from left to right to appropriate new providers
// - user can correct mistakes by dragging patients between new providers
// - when assignments are complete, user commands program to apply changes
// - user optionally repeats this process for another provider

// *******************************************************************
// Programmer notes...
// *******************************************************************
// - Created May 2017, Scott Stephenson (Delphi) and Robert Davis (M)
// - NSR 20130504, originally targeted for CPRS EP1
// - This is a client application normally called from CPRS Tools menu, but optionally standalone
// - Typical user is a Clinical Applications Coordinator
// - See Utilities.pas for a list of RPCs used by this application
// - This application supports the last RPC call window - hit Ctrl-F1 to see it

// GUI things a maintenance programmer should know...
// - in all nodes, stringData stores PatientIEN^PatientName
// - when searching for patient or provider, we use IEN, because name may not be unique
// - drag/drop is supplemented by transfer buttons for accessibility
// - we use double-click and context menus
// - all actions are done from actions stored in an action list
// - enabled states are centrally set in the action list's onUpdate event

// To add a CPRS user menu item for this application, use...
// Provider Role Tool=SomePath\ProviderRoleTool.exe S=%SERVER P=%PORT
// See explanation at: https://www.domain/vdl/documents/Clinical/Comp_Patient_Recrd_Sys_(CPRS)/cprsguitm.pdf

interface

uses
  // Application units...
  uProviderEngine,
  fConnectToServer,
  fApplyChanges,
  uUtilities,
  fBase508Form,
  VA508AccessibilityManager,
  ORCtrls,
  ORDtTmRng,
  // Delphi units...
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls,
  Vcl.ComCtrls,
  System.Actions, System.UITypes, System.Types, Vcl.ActnList,
  System.DateUtils, Vcl.Menus, Vcl.StdActns,
  System.Generics.Collections, System.Generics.Defaults, Winapi.commctrl,
  System.Win.TaskbarCore, Vcl.Taskbar,
  //Needs to be last for the button extension
  U508Extensions;

type

  TFormReassignOrders = class(TfrmBase508Form)
    pnlLeft: TPanel;
    pnlMiddle: TPanel;
    pnlRight: TPanel;
    btnAddSelected: U508Extensions.TButton;
    BtnAddAll: U508Extensions.TButton;
    BtnUndoAll: U508Extensions.TButton;
    BtnUndoSelected: U508Extensions.TButton;
    ActionsMain: TActionList;
    ActionAddSelected: TAction;
    ActionAddAll: TAction;
    ActionUndoSelected: TAction;
    ActionUndoAll: TAction;
    btnCurrentProvider: U508Extensions.TButton;
    ActionChooseCurrentProvider: TAction;
    ButtonAddNewProvider: U508Extensions.TButton;
    ActionAddNewProvider: TAction;
    ActionApplyChanges: TAction;
    btnApplyChanges: U508Extensions.TButton;
    tvNewAssignments: TORTreeView;
    pmCurrent: TPopupMenu;
    pmNew: TPopupMenu;
    ActionExpandAll: TAction;
    popNewAssignmentsFullExpand: TMenuItem;
    ActionCollapseAll: TAction;
    popNewAssignmentsFullCollapse: TMenuItem;
    ActionShowPatientOrders: TAction;
    popShowCurrentPatientOrders: TMenuItem;
    MainMenuPU: TMainMenu;
    mnuFile: TMenuItem;
    mnuEdit: TMenuItem;
    mnuHelpTitle: TMenuItem;
    MnuExit: TMenuItem;
    mnuSeparator1: TMenuItem;
    ActionExit: TAction;
    MnuChooseCurrentProviderandOrderDates: TMenuItem;
    MnuAddNewProvider: TMenuItem;
    MnuApplyChanges: TMenuItem;
    MnuAddSelected: TMenuItem;
    MnuAddAll: TMenuItem;
    MnuUndoSelected: TMenuItem;
    MnuUndoAll: TMenuItem;
    ActionAbout: TAction;
    MnuAbout: TMenuItem;
    lblPatientCount: TVA508StaticText;
    ActionAutoAllocate: TAction;
    ActionSelectAll: TAction;
    popCurrentSelectAll: TMenuItem;
    ActionUnselectAll: TAction;
    N1: TMenuItem;
    popCurrentDeselectAll: TMenuItem;
    pnlTopMiddleSpacer: TPanel;
    mnuAutoAllocateAll: TMenuItem;
    mnuHelp: TMenuItem;
    popShowNewPatientOrders: TMenuItem;
    N2: TMenuItem;
    ActionRPCLog: TAction;
    mnuRPCLog: TMenuItem;
    N3: TMenuItem;
    actionHelpContents: THelpContents;
    lblCurrentProviderPatients: TVA508StaticText;
    PnlLftTp: TPanel;
    PnlLftBtm: TPanel;
    PnlRghtTp: TPanel;
    PnlRghtBtm: TPanel;
    lblNewPatientProviders: TVA508StaticText;
    N4: TMenuItem;
    popNewSelectAll: TMenuItem;
    popNewUnselectAll: TMenuItem;
    gpMain: TGridPanel;
    lvCurrentAssignments: TListView;
    N5: TMenuItem;
    AuditReport1: TMenuItem;
    actionAuditReport: TAction;
    gpOrders: TGridPanel;
    pnlPendNone: TPanel;
    pnlOut: TPanel;
    lnlPendOut: TLabel;
    lblPendNone: TLabel;
    lvPendNone: TListView;
    lvPendOut: TListView;
    lvPatient: TListView;
    View1: TMenuItem;
    viewAll: TMenuItem;
    ViewPat: TMenuItem;
    viewOrd: TMenuItem;
    ActionViewPat: TAction;
    ActionViewXfer: TAction;
    ActionViewOrd: TAction;
    HeaderControl1: THeaderControl;
    ScrollBox1: TScrollBox;
    Panel3: TPanel;
    pnlIn: TPanel;
    lblPendIn: TLabel;
    lvPendIn: TListView;
    lblOutTotal: TLabel;
    lblInTotal: TLabel;
    lblNoneTotal: TLabel;
    procedure ActionsMainUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure tvNewAssignmentsDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure tvNewAssignmentsDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ActionAddSelectedExecute(Sender: TObject);
    procedure ActionAddAllExecute(Sender: TObject);
    procedure ActionUndoSelectedExecute(Sender: TObject);
    procedure ActionUndoAllExecute(Sender: TObject);
    procedure ActionChooseCurrentProviderExecute(Sender: TObject);
    procedure ActionAddNewProviderExecute(Sender: TObject);
    procedure ActionApplyChangesExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MessageConnectToServer(Var Msg: TMessage);
      message MSG_CONNECT_TO_SERVER;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ActionExpandAllExecute(Sender: TObject);
    procedure ActionCollapseAllExecute(Sender: TObject);
    procedure ActionShowPatientOrdersExecute(Sender: TObject);
    procedure ActionExitExecute(Sender: TObject);
    procedure ActionAboutExecute(Sender: TObject);
    procedure ActionSelectAllExecute(Sender: TObject);
    procedure ActionUnselectAllExecute(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure pmCurrentPopup(Sender: TObject);
    procedure pmNewPopup(Sender: TObject);
    procedure tvNewAssignmentsDblClick(Sender: TObject);
    procedure ActionRPCLogExecute(Sender: TObject);
    procedure actionHelpContentsExecute(Sender: TObject);
    procedure lvDeletion(Sender: TObject; Item: TListItem);
    procedure lvDblClick(Sender: TObject);
    procedure lvDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lvDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure actionAuditReportExecute(Sender: TObject);
    procedure ActionViewPatExecute(Sender: TObject);
    procedure tvNewAssignmentsDeletion(Sender: TObject; Node: TTreeNode);
    procedure HeaderControl1SectionTrack(HeaderControl: THeaderControl;
      Section: THeaderSection; Width: Integer; State: TSectionTrackState);
    procedure tvNewAssignmentsAdvancedCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; Stage: TCustomDrawStage;
      var PaintImages, DefaultDraw: Boolean);
    procedure tvNewAssignmentsChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure ScrollBox1Resize(Sender: TObject);
    procedure lvPendSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure pnlRightResize(Sender: TObject);
    procedure LVOnDataData(Sender: TObject; Item: TListItem);
    procedure FormCreate(Sender: TObject);
    procedure tvNewAssignmentsStartDrag(Sender: TObject;
      var DragObject: TDragObject);
  private
    fConnectionFrame: TConnectionProgress;
    fOrderEngine: TOrderEngine;
    fTreeView_Deleting: Boolean;
    fViewType: tViewType;
    fLVDict: tSrchDict;
    fLVGDict: tSrchDictGroup;
    fPSB: tProgressStatusBar;
    fTransferOrdersLst: tTransferOders;
    fAutoSizedCols: Boolean;
    fTV508Manager: Ttv508Manager;
    fLV508Manager: Tlv508Manager;
    fDropLocations: tDropLocations;
    function TimeOutCondition: Boolean;
    procedure TimeOutAction;
    procedure AddNewProviderToList(ANewProvider: tPersonRecord);
    function ProviderExistsInTreeList(AProviderIEN: string): TTreeNode;
    function IsThereUnsavedWork: Boolean;
    procedure PopulateAllPatientsLists;
    Procedure PopulatePatientList(const Orders: tOrderRecords);
    Procedure PopulateXferList(const Orders: tOrderRecords);
    Procedure PopulateOrderList(const Orders: tOrderRecords);
    procedure RemoveItemsFromList(SrcObj: TObject);
    Procedure MoveItemsBetweenList(DestLst, SrcObj: TObject;
      AParentNode: TTreeNode = nil); overload;
    procedure UpdatePatientsLabel;
    function PrepareReassignments: TReassignOrderRecords;
    function CheckAndQueryUnsavedWork: Boolean;
    procedure UpdateScreenAfterReassignment;
    procedure ShowPatientCount;
    function NodeToPerson(ANode: TORTreeNode): tPersonRecord;
    procedure selectAll(AView: TObject; Select: Boolean);
    procedure InitializeTimeoutFunctionality;
    procedure ShowPatientDetails(AHostObj: TObject;
      const aFromIEN: string = '');
    Procedure LoadExceptionLogger;
    procedure ActivateMenu(const State: Boolean);
    Function GetPopUpControl(Sender: TObject): TComponent;
    procedure SetHeaderSize(aTreeNode: TTreeNode = nil);
    Procedure UpdateLinkedListViews();
    Function CanMoveToProvider(SrcObj: TObject; ToProvider: TTreeNode): Boolean;
    procedure SetColumnWidth();
    function PendingChangesDlg: Integer;
    function ConstainsObj(X: TListView; anArray: tDropLocations): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property ProgressStatusBar: tProgressStatusBar read fPSB;
    Property TransferOrders: tTransferOders read fTransferOrdersLst;
    Property OrderEngine: TOrderEngine read fOrderEngine;
  end;

var
  FormReassignOrders: TFormReassignOrders;

implementation

{$R *.dfm}

uses
  fAbout, // form to view program information
  fSelectProvider, // form to select a current or new provider
  fPatientOrders, // form to view patient order details
  uGN_RpcLog, // functions controlling form to view RPC calls
  uInit,
  orNet,
  shellApi,
  orFn,
  VAUtils,
  u_HelpMGR,
  ORCtrlsVA508Compatibility,
  AVCatcher,
  uSizeEngine,
  faudit,
  System.Math,
  fPatCheck;

const
  ID_IEN = 1; // for a node stringData, grab piece 1 for IEN
  ID_NAME = 2; // for a node stringData, grab piece 2 for Patient Name
  ID_PROVIDER = 3; // for a node stringData, grab piece 3 for Is Provider

  // CB
procedure TFormReassignOrders.ActivateMenu(const State: Boolean);
// *******************************************************************
// Sets up the menu and border icons
// *******************************************************************
var
  aMenu: TMenuItem;
begin
  // Show the proper border icons when we are connected
  if not State then
  begin
    self.BorderIcons := [biSystemMenu];
    self.BorderStyle := bsSingle;
  end
  else
  begin
    self.BorderIcons := [biSystemMenu, biMinimize, biMaximize];
    self.BorderStyle := bsSizeable;
  end;

  // Enable the menu if we are connected
  for aMenu in MainMenuPU.Items do
  begin
    if State then
      EnableMenuItem(MainMenuPU.Handle, aMenu.Command, MF_BYCOMMAND or
        MF_ENABLED)
    else
      EnableMenuItem(MainMenuPU.Handle, aMenu.Command, MF_BYCOMMAND or
        MF_DISABLED);
  end;
end;

// CB
Procedure TFormReassignOrders.LoadExceptionLogger;
// *******************************************************************
// Loas up the exception catcher
// *******************************************************************
var
  TmpLst: TStringList;
  TmpStr: String;
begin
  TmpLst := TStringList.Create;
  try
    ExceptionLog.DaysToPurge :=
      StrToIntDef(GetUserParam('PRT EXCEPTION PURGE'), 60);
    ExceptionLog.Enabled :=
      Uppercase(GetUserParam('PRT EXCEPTION LOGGER')) = '1';
    GetUserListParam(TmpLst, 'PRT EXCEPTION EMAIL');
    for TmpStr in TmpLst do
      ExceptionLog.EmailTo.Add(Piece(TmpStr, U, 2));
  finally
    TmpLst.Free;
  end;
end;

// CB
procedure TFormReassignOrders.lvDblClick(Sender: TObject);
// *******************************************************************
// Display the details when double clicked
// *******************************************************************
begin
  // Grab the first selected order when double clicking
  if (Sender is TListView) and (TListView(Sender).SelCount = 0) then
    exit;
  ShowPatientDetails(Sender);
end;

// CB
procedure TFormReassignOrders.LVOnDataData(Sender: TObject; Item: TListItem);
// *******************************************************************
// Pull the data for the virtual list view
// *******************************************************************
var
  aOrder: tOrderRecord;
begin
  inherited;
  // Grab the order from our list
  if Sender = lvPendOut then
    aOrder := fOrderEngine.OrdersPendingOut[Item.Index]
  else if Sender = lvPendNone then
    aOrder := fOrderEngine.OrdersPendingNone[Item.Index]
  else if Sender = lvPendIn then
    aOrder := fOrderEngine.OrdersPendingIn[Item.Index];
  // populate the item based on the order
  Item.Caption := aOrder.patient.name;
  Item.SubItems.Add(aOrder.OrderIEN);
  Item.SubItems.Add(aOrder.orderDate);
  Item.SubItems.Add(aOrder.orderStatus);
  Item.SubItems.Add(FormatFMDateTimeStr('mm/dd/yy@hh:nn', aOrder.orderXferEffective));
  Item.SubItems.Add(aOrder.orderXferFrom.name);
end;

// CB
procedure TFormReassignOrders.lvPendSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
// *******************************************************************
// Global onSelectItem. Used for the Virtual ListViews
// *******************************************************************
begin
  inherited;
  UpdateLinkedListViews;
end;

// CB
procedure TFormReassignOrders.lvDeletion(Sender: TObject; Item: TListItem);
// *******************************************************************
// ListView Deletion event
// *******************************************************************
begin
  TListView(Sender).CleanUpListItem(Item);
end;

// CB
procedure TFormReassignOrders.lvDragDrop(Sender, Source: TObject;
  X, Y: Integer);
// *******************************************************************
// ListView drop event
// *******************************************************************
begin
  inherited;
  // Undo any selections
  ActionUndoSelectedExecute(Sender);
end;

// CB
procedure TFormReassignOrders.lvDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
// *******************************************************************
// ListView drop event
// *******************************************************************
begin
  inherited;
  // accept the drop if the source is new patients list box
  Accept := (Source = tvNewAssignments);
  if Accept then
  begin
    if Sender is tListView then
    begin
      Accept := ConstainsObj(tListView(Sender), fDropLocations);
    end;
  end;
end;

// CB
Procedure TFormReassignOrders.UpdateLinkedListViews();
// *******************************************************************
// Ensure that only one list view has selected items
// *******************************************************************
begin
  if lvPendNone.Focused then
  begin
    if lvPendOut.SelCount > 0 then
      lvPendOut.ClearSelection;
    if lvPendIn.SelCount > 0 then
      lvPendIn.ClearSelection;
  end;

  if lvPendOut.Focused then
  begin
    if lvPendNone.SelCount > 0 then
      lvPendNone.ClearSelection;
    if lvPendIn.SelCount > 0 then
      lvPendIn.ClearSelection;
  end;

  if lvPendIn.Focused then
  begin
    if lvPendNone.SelCount > 0 then
      lvPendNone.ClearSelection;
    if lvPendOut.SelCount > 0 then
      lvPendOut.ClearSelection;
  end;
end;

// CB
constructor TFormReassignOrders.Create(AOwner: TComponent);
// *******************************************************************
// override default constructor to initialize order engine, etc
// *******************************************************************
begin
  inherited Create(AOwner);
  ActivateMenu(false);
  // make a connection frame to show the user we're connecting to the server
  fConnectionFrame := TConnectionProgress.Create(self, amgrMain);
  fConnectionFrame.parent := self;
  fConnectionFrame.align := alClient;
  fConnectionFrame.visible := true;

  // initialize the form
  fOrderEngine := TOrderEngine.Create;
  // Setup
  fLVDict := TDictionary<String, TListItem>.Create(1009);
  fLVGDict := TDictionary<String, TListGroup>.Create(10009);
  SizeHolder := TSizeHolder.Create;
  UpdatePatientsLabel;
  // Create status bar with progress
  fPSB := tProgressStatusBar.Create(self);
  fPSB.visible := false;
  fPSB.DisplayHints := true;

  if ScreenReaderActive then
  begin
    fTV508Manager := Ttv508Manager.Create(HeaderControl1, [4,5,6,7,8], True, 'Transfer To');
    amgrMain.ComponentManager[tvNewAssignments] := fTV508Manager;

    fLV508Manager := Tlv508Manager.Create;
    amgrMain.ComponentManager[lvCurrentAssignments] := fLV508Manager;
    amgrMain.ComponentManager[lvPendNone] := fLV508Manager;
    amgrMain.ComponentManager[lvPendOut] := fLV508Manager;
    amgrMain.ComponentManager[lvPendIn] := fLV508Manager;
    amgrMain.ComponentManager[lvPatient] := fLV508Manager;
  end;

end;

// CB
destructor TFormReassignOrders.Destroy;
// *******************************************************************
// destroy our order engine so no memory leak is reported
// *******************************************************************
begin
  //Clean up 508 managers
  if assigned(fLV508Manager) then
  begin
    amgrMain.ComponentManager[lvCurrentAssignments] := nil;
    amgrMain.ComponentManager[lvPendNone] := nil;
    amgrMain.ComponentManager[lvPendOut] := nil;
    amgrMain.ComponentManager[lvPendIn] := nil;
    amgrMain.ComponentManager[lvPatient] := nil;
    FreeAndNil(fLV508Manager);
  end;

  if assigned(fTV508Manager) then
  begin
    amgrMain.ComponentManager[tvNewAssignments] := nil;
    FreeAndNil(fTV508Manager);
  end;

  FreeAndNil(fOrderEngine);
  FreeAndNil(SizeHolder);
  FreeAndNil(fLVDict);
  FreeAndNil(fLVGDict);
  FreeAndNil(fPSB);
  inherited Destroy;
end;

function TFormReassignOrders.PrepareReassignments: TReassignOrderRecords;
// *******************************************************************
// we iterate the reassigned patients in the tree list, scan orders
// for each, and reassign
// *******************************************************************
var
  aOldOrder: tOrderRecord;
  newOrderID: Integer;
  aPatientOrders: tOrderRecords;
  ANewProvider: tPersonRecord;
  ANode: TORTreeNode;
  nodeEnumerator: TTreeNodesEnumerator;
  // aXferRec: pXferToRec;
  XferRecord: tTransferToRec;
  rtnCsr: TCursor;
begin
  rtnCsr := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    ProgressStatusBar.ResetProgressBar(tvNewAssignments.Items.Count);
    ProgressStatusBar.ResetLoop(tvNewAssignments.Items.Count);
    ANewProvider.Clear;
    newOrderID := 0;
    setLength(result, length(fOrderEngine.Orders));
    // biggest it can possibly be
    nodeEnumerator := TTreeNodesEnumerator.Create(tvNewAssignments.Items);
    try
      while nodeEnumerator.moveNext do
      begin
        ANode := TORTreeNode(nodeEnumerator.Current);
        if ANode.hasChildren then
        begin
          // this is not a patient node, so just retain the new provider node and continue
          ANewProvider := NodeToPerson(ANode);
        end
        else if ANode.level = 1 then
        begin
          XferRecord := fTransferOrdersLst.TransferRec[ANode];
          // aXferRec := pXferToRec(ANode.Data);

          // case XferRecord.ListObject.Group_Type of
          case XferRecord.Group_Type of
            tvtPatient:
              begin
                aPatientOrders := fOrderEngine.ordersForPatient
                  (XferRecord.PatientInfo.IEN);
              end;
            tvtXfer:
              begin
                // aOldOrder := fOrderEngine.OrderByIEN(XferRecord.ListObject.orderIEN);
                aPatientOrders := fOrderEngine.ordersForPatientAndXfer
                  (XferRecord.PatientInfo.IEN,
                  XferRecord.PatientInfo.OrderXfer);
              end;
            tvtOrder:
              begin
                setLength(aPatientOrders, 1);
                aPatientOrders[0] := fOrderEngine.OrderByIEN
                  (XferRecord.{ ListObject. } OrderIEN);
              end;
          end;

          for aOldOrder in aPatientOrders do
          begin
            result[newOrderID].requesterIEN := rpcbrokerv.User.DUZ;
            result[newOrderID].patientIEN := XferRecord.PatientInfo.IEN;
            result[newOrderID].patientName := XferRecord.PatientInfo.name;
            result[newOrderID].oldProviderIEN :=
              fOrderEngine.CurrentProvider.IEN;
            result[newOrderID].oldProviderName :=
              fOrderEngine.CurrentProvider.name;
            result[newOrderID].newProviderIEN := XferRecord.TransferTo.IEN;
            result[newOrderID].newProviderName := XferRecord.TransferTo.name;
            result[newOrderID].effectiveDateTime := '';
            // meaningless - will be reassigned later
            result[newOrderID].OrderIEN := aOldOrder.OrderIEN;
            case aOldOrder.orderFlag of
              tofNA: result[newOrderID].orderFlag := '';
              tofCancel: result[newOrderID].orderFlag := 'C';
              tofOverwrite: result[newOrderID].orderFlag := 'O';
            end;
            inc(newOrderID);
          end;

        end;
        ProgressStatusBar.StepProgress;
        if ProgressStatusBar.CancelClicked then
          break;
      end;
      setLength(result, newOrderID);
      ProgressStatusBar.HideProgress;
    finally
      freeAndNil(nodeEnumerator);
    end;
  finally
    screen.Cursor := rtnCsr;
  end;
end;

procedure TFormReassignOrders.MessageConnectToServer(Var Msg: TMessage);
// We receive this message after the main form is visible
// Let the user know we're doing something by showing a connection note
// hide the note and show the page control only if we connect
const
  fmtConnect = '%s -- %s : %d';

begin
  if fConnectionFrame.EstablishConnection then
  begin
    pnlLeft.visible := true;
    pnlMiddle.visible := true;
    pnlRight.visible := true;
    InitializeTimeoutFunctionality;
    // only if we connected (a user is logged into the system)

    // sbMain.Panels[0].Text :=
    // Format(fmtConnect,[RPCBrokerV.User.StandardName, RPCBrokerV.Server, RPCBrokerV.ListenerPort]);
    ProgressStatusBar.visible := true;
    if assigned(fConnectionFrame) then
      fConnectionFrame.Free;
    LoadExceptionLogger;
    ActivateMenu(true);
    // Load the sizes
    SizeHolder.LoadAllSizes;
    SizeHolder.LoadSize(self);
    fPSB.visible := true;
  end
  else
    Application.Terminate;
end;

// CB
procedure TFormReassignOrders.FormClose(Sender: TObject;
  var Action: TCloseAction);
// ******************************************************************************
// Perform any cleanup and save last positions
// ******************************************************************************
begin
  // Save all the sizes
  SizeHolder.SetSize(self);
  SizeHolder.SaveAllSizes;
  inherited;
end;

// CB
procedure TFormReassignOrders.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
// ******************************************************************************
// allow close if there are no changes, or user says Ok to lose changes
// ******************************************************************************
begin
  CanClose := CheckAndQueryUnsavedWork;
end;

procedure TFormReassignOrders.FormCreate(Sender: TObject);
begin
  inherited;

end;

// CB
procedure TFormReassignOrders.FormResize(Sender: TObject);
// ******************************************************************************
// this prevents controls from disappearing on form resizing
// ******************************************************************************
begin
  // inherited;
  // first we need to add a few runtime size constraints
  pnlMiddle.Constraints.MinHeight := BtnUndoAll.Top + BtnUndoAll.Height + 10;
  pnlLeft.Constraints.MinWidth := trunc(pnlMiddle.Width * 1.5);
  pnlRight.Constraints.MinWidth := pnlLeft.Constraints.MinWidth;
end;

// CB
procedure TFormReassignOrders.FormShow(Sender: TObject);
// ******************************************************************************
// Post a message to ourselves to connect after the main form is shown
// We do this so the user has something to look at without needing a
// splash screen
// ******************************************************************************
begin
  PostMessage(self.Handle, MSG_CONNECT_TO_SERVER, 0, 0);
end;

// CB
function TFormReassignOrders.IsThereUnsavedWork: Boolean;
// ******************************************************************************
// Figure out if there is any unsaved transfers.
// Provider(s) must have orders under them
// ******************************************************************************
var
  nodeEnumerator: TTreeNodesEnumerator;
begin
  // set the default
  result := false;
  // Loop through the potential provider(s)
  nodeEnumerator := TTreeNodesEnumerator.Create(tvNewAssignments.Items);
  try
    while nodeEnumerator.moveNext do
      // If this isn't at the base level then its an order
      if nodeEnumerator.Current.level > 0 then
      begin
        result := true;
        break;
      end;
  finally
    freeAndNil(nodeEnumerator);
  end;
end;

// CB
procedure TFormReassignOrders.UpdatePatientsLabel;
// ******************************************************************************
// display information on the patients and orders retrieved from the server
// ******************************************************************************
begin
  lblCurrentProviderPatients.Caption := 'Patients with qualifying orders' +
    sLineBreak + format('Provider: %s', [fOrderEngine.CurrentProvider.name]) +
    sLineBreak + format('Orders: %s to %s', [DateToStr(fOrderEngine.DateStart),
    DateToStr(fOrderEngine.DateStop)]);
end;

procedure TFormReassignOrders.ActionViewPatExecute(Sender: TObject);

  procedure ClearChildren;
  var
    ANode: TTreeNode;
  begin

    if tvNewAssignments.Items.Count = 0 then
      exit;

    ANode := tvNewAssignments.Items[0];
    while assigned(ANode) do
    begin
      if ANode.hasChildren then
      begin
        ANode.DeleteChildren;
      end;
      ANode := ANode.getNextSibling;
    end;
  end;

  Procedure UpdateHeader;
  const
    Group_Header: Array [1 .. 2] of string = ('Patient Name',
      'Number of Orders');
    Order_Header: Array [1 .. 4] of string = ('Patient Name', 'Order IEN',
      'Order Date', 'Order Status');
  var
    HeaderSection: THeaderSection;
    ColText: String;
  begin
    if fViewType = tvtOrder then
    begin
      HeaderControl1.Sections.Clear;
      for ColText in Order_Header do
      begin
        HeaderSection := HeaderControl1.Sections.Add;
        HeaderSection.Text := ColText;
      end;
    end
    else
    begin
      HeaderControl1.Sections.Clear;
      for ColText in Group_Header do
      begin
        HeaderSection := HeaderControl1.Sections.Add;
        HeaderSection.Text := ColText;
      end;
    end;
    SetHeaderSize;
  end;

begin
  if not(Sender is TAction) then
    exit;

  if not CheckAndQueryUnsavedWork then
  begin
      case fViewType of
        tvtPatient:
          ActionViewPat.Checked := true;
        tvtXfer:
          ActionViewXfer.Checked := true;
        tvtOrder:
          ActionViewOrd.Checked := true;
      end;
      UpdateHeader;
      exit;
  end;

  if tvNewAssignments.Items.Count > 0 then
    ClearChildren;

  If TAction(Sender) = ActionViewPat then
  begin
    fViewType := tvtPatient;
    lvCurrentAssignments.visible := true;
    gpOrders.visible := false;
    lvPatient.visible := false;
  end
  else If TAction(Sender) = ActionViewXfer then
  begin
    fViewType := tvtXfer;
    lvPatient.visible := true;
    lvCurrentAssignments.visible := false;
    gpOrders.visible := false;
  end
  else If TAction(Sender) = ActionViewOrd then
  begin
    fViewType := tvtOrder;
    lvCurrentAssignments.visible := false;
    gpOrders.visible := true;
    lvPatient.visible := false;
  end;

  PopulateAllPatientsLists;
  UpdateHeader;
end;

procedure TFormReassignOrders.ActionAboutExecute(Sender: TObject);
begin
  ShowAbout;
end;

procedure TFormReassignOrders.ActionAddAllExecute(Sender: TObject);
// move all patients to the selected new provider
var
  rtnCsr: TCursor;
begin
  if (tvNewAssignments.Selected <> nil) and (tvNewAssignments.Selected.level = 0)
  then
  begin
    rtnCsr := screen.Cursor;
    screen.Cursor := crHourGlass;
    try
      if ActionViewPat.Checked then
      begin
        lvCurrentAssignments.selectAll;
        ProgressStatusBar.ResetProgressBar(lvCurrentAssignments.SelCount * 3);
        MoveItemsBetweenList(tvNewAssignments, lvCurrentAssignments,
          tvNewAssignments.Selected);
        RemoveItemsFromList(lvCurrentAssignments);
        ProgressStatusBar.HideProgress;
      end
      else if ActionViewXfer.Checked then
      begin
        lvPatient.selectAll;
        ProgressStatusBar.ResetProgressBar(lvPatient.SelCount * 3);
        MoveItemsBetweenList(tvNewAssignments, lvPatient,
          tvNewAssignments.Selected);
        RemoveItemsFromList(lvPatient);
        ProgressStatusBar.HideProgress;
      end
      else if ActionViewOrd.Checked then
      begin
        ProgressStatusBar.ResetProgressBar((lvPendNone.Items.Count * 3) +
          (lvPendOut.Items.Count * 3) + (lvPendIn.Items.Count * 3));
        lvPendNone.selectAll;
        MoveItemsBetweenList(tvNewAssignments, lvPendNone,
          tvNewAssignments.Selected);
        RemoveItemsFromList(lvPendNone);
        lvPendOut.selectAll;
        MoveItemsBetweenList(tvNewAssignments, lvPendOut,
          tvNewAssignments.Selected);
        RemoveItemsFromList(lvPendOut);
        lvPendIn.selectAll;
        MoveItemsBetweenList(tvNewAssignments, lvPendIn,
          tvNewAssignments.Selected);
        RemoveItemsFromList(lvPendIn);
        ProgressStatusBar.HideProgress;
      end;
    finally
      screen.Cursor := rtnCsr;
    end;
  end;
end;

procedure TFormReassignOrders.ActionAddSelectedExecute(Sender: TObject);
// move only selected patients to the new provider
var
  // Patients: tPersonRecords;
  SrcObj: TListView;
  rtnCsr: TCursor;
begin
  if (tvNewAssignments.Selected <> nil) and (tvNewAssignments.Selected.level = 0)
  then
  begin
    rtnCsr := screen.Cursor;
    screen.Cursor := crHourGlass;
    try
      SrcObj := nil;
      if ActionViewPat.Checked then
        SrcObj := lvCurrentAssignments
      else if ActionViewXfer.Checked then
        SrcObj := lvPatient
      else if ActionViewOrd.Checked then
      begin
        if lvPendNone.SelCount > 0 then
          SrcObj := lvPendNone
        else if lvPendOut.SelCount > 0 then
          SrcObj := lvPendOut
        else if lvPendIn.SelCount > 0 then
          SrcObj := lvPendIn;
      end;

      if not assigned(SrcObj) then
        exit;

      ProgressStatusBar.ResetProgressBar(TListView(SrcObj).SelCount * 3);
      MoveItemsBetweenList(tvNewAssignments, SrcObj, tvNewAssignments.Selected);
      RemoveItemsFromList(SrcObj);
      ProgressStatusBar.HideProgress;

      //We are adding to the tree so ensure eveything is visible
      if srcObj is TListView then
        ScrollBox1Resize(self);
    finally
      screen.Cursor := rtnCsr;
    end;
  end;
end;

Function TFormReassignOrders.CanMoveToProvider(SrcObj: TObject;
  ToProvider: TTreeNode): Boolean;
type
  tPatDict = TDictionary<TObject, Integer>;
var
  fPatDict: tPatDict;

  Function FindPatRec(aLookupObj: TObject; aPatArry: tPatRecArry): Integer;
  begin
    if not fPatDict.TryGetValue(aLookupObj, result) then
      result := -1;
  end;

// ********************************************************
// Check to see if the patient has transfers
// ********************************************************
  procedure CheckForIssues(aSelObj: TObject; aLV: TListView;
    aProvObj: tPersonRecord; var aPatArry: tPatRecArry; var OtherOrders: Boolean);
  var
    aOrRec: tOrderRecord;
    aOrRecLst: tOrderRecords;
    CurrentRec: Integer;
    aLvItm: TListItem;
    XferRecord: tTransferToRec;
    aLstObj: tListObject;
  begin
    // Setup
    if aSelObj is TListItem then
    begin
      aLvItm := TListItem(aSelObj);
      // If this is from our OnData list then treat this differently
      if (aLV = lvPendNone) or (aLV = lvPendOut) or (aLV = lvPendIn) then
      begin
        // Grab the order based on the selection
        if aLV = lvPendNone then
          aOrRec := fOrderEngine.OrdersPendingNone[aLvItm.Index]
        else if aLV = lvPendOut then
          aOrRec := fOrderEngine.OrdersPendingOut[aLvItm.Index]
        else if aLV = lvPendIn then
          aOrRec := fOrderEngine.OrdersPendingIn[aLvItm.Index];
        // We only want to check this one order
        setLength(aOrRecLst, 1);
        aOrRecLst[0] := aOrRec;
      end
      else
      begin
        // Must have an object attached to it
        if not assigned(aLvItm.Data) then
          exit;
        // Use the attached object to figure out what order this is
        aLstObj := tListObject(aLvItm.Data^);
        aOrRec := fOrderEngine.OrderByIEN(aLstObj.OrderIEN);

        // Grab the orders that match our view type
        if aLstObj.Group_Type = tvtXfer then
          aOrRecLst := fOrderEngine.ordersForPatientAndXfer(aOrRec.patient.IEN,
            aOrRec.OrderXfer)
        else if aLstObj.Group_Type = tvtPatient then
          aOrRecLst := fOrderEngine.ordersForPatient(aOrRec.patient.IEN);

      end;
    end
    else if aSelObj is TTreeNode then
    begin

      // From the Tree so make sure it has some data
      if not assigned(TTreeNode(aSelObj).Data) then
        exit;

      // Grab the data object and load the order from it
      XferRecord := fTransferOrdersLst.TransferRec[TTreeNode(aSelObj)];

      aOrRec := fOrderEngine.OrderByIEN(XferRecord.OrderIEN);

      // Get the orders that will be checked based on the view type
      if XferRecord.Group_Type = tvtOrder then
      begin
        setLength(aOrRecLst, 1);
        aOrRecLst[0] := aOrRec;
      end
      else if XferRecord.Group_Type = tvtXfer then
        aOrRecLst := fOrderEngine.ordersForPatientAndXfer(aOrRec.patient.IEN,
          aOrRec.OrderXfer)
      else if XferRecord.Group_Type = tvtPatient then
        aOrRecLst := fOrderEngine.ordersForPatient(aOrRec.patient.IEN);
    end;

    // default as not exisiting yet
    OtherOrders := false;

    for aOrRec in aOrRecLst do
    begin
      // If we have an order with transfer history
      if (aOrRec.OrderXfer <> txftNone) and (aOrRec.OrderXfer <> txftTransferred) then
      begin
          // Find our patient object if it exist
          CurrentRec := FindPatRec(aSelObj, aPatArry);
          // If we have one in the list but are moving between the same user
          if (CurrentRec <> -1) and (aPatArry[CurrentRec].Reason = txrSame) then
            exit;


          // No patient object so we need to add one
          if CurrentRec = -1 then
          begin
            setLength(aPatArry, length(aPatArry) + 1);
            CurrentRec := High(aPatArry);
            aPatArry[CurrentRec].SelObj := aSelObj;
            // As long as we are not looking at all orders grouped together
            if fViewType <> tvtOrder then
            begin
              aPatArry[CurrentRec].name := aOrRec.patient.name;
              aPatArry[CurrentRec].IEN := aOrRec.patient.IEN;
            end else if fViewType = tvtOrder then
            begin
              aPatArry[CurrentRec].name := Format('%s^%s^%s^%s',
              [aOrRec.patient.name, aOrRec.orderIEN, aOrRec.orderXferFrom.name, FormatFMDateTimeStr('mm/dd/yy@hh:nn', aOrRec.orderXferEffective)]);
              aPatArry[CurrentRec].IEN := aOrRec.orderIEN;
            end;
            aPatArry[CurrentRec].Reason := txrNone;
            fPatDict.Add(aSelObj, CurrentRec);
          end;



        //If orders are truly pending
        if StrToFloatDef(aOrRec.orderXferEffective, 0) > FMNow then
        begin
          begin
            if (aOrRec.orderXferFrom.IEN = aProvObj.IEN) and (aOrRec.OrderXfer = txftPendIn) then
            begin
              aPatArry[CurrentRec].Reason := txrCancel;
              fOrderEngine.UpdateOrderFlagByOrder(tofCancel, aOrRec.orderIEN);
            {end else if (aProvObj.IEN = fOrderEngine.CurrentProvider.IEN)  and (aOrRec.OrderXfer = txftPendOut)}
            end else if (aOrRec.orderXferFrom.IEN = aProvObj.IEN) and (aOrRec.OrderXfer = txftPendOut) then
            begin
              aPatArry[CurrentRec].Reason := txrSame;
             // fOrderEngine.UpdateOrderFlagByOrder(tofCancel, aOrRec.orderIEN);
            end else if not (aSelObj is TTreeNode) then begin
              aPatArry[CurrentRec].Reason := txrPend;
              fOrderEngine.UpdateOrderFlagByOrder(tofOverwrite, aOrRec.orderIEN);
            end;

          end;
        end else
          OtherOrders := true;
      end else
        OtherOrders := true;
    end;
  end;

var
  ProvObj: tPersonRecord;
  I: Integer;
  aLV: TListView;
  aTr: TTreeView;
  PatLst: tPatRecArry;
  ChkRst: TModalResult;
  NonXferOrders, TmpBool: Boolean;
begin
  // Assume we are good
  result := true;
  NonXferOrders := false;
//  if fViewType = tvtOrder then
//    exit;
  fPatDict := TDictionary<TObject, Integer>.Create(10009);
  try
    // Who is this going to
    ProvObj := NodeToPerson(TORTreeNode(ToProvider));
    setLength(PatLst, 0);
    if SrcObj is TListView then
    begin
      aLV := TListView(SrcObj);
      ProgressStatusBar.ResetLoop(aLV.SelCount);
      for I := 0 to aLV.Items.Count - 1 do
      begin
        if aLV.Items[I].Selected then
        begin
          // check each order to see if we can move this over
          CheckForIssues(aLV.Items[I], aLV, ProvObj, PatLst, TmpBool);
          if TmpBool then
            NonXferOrders := true;
          ProgressStatusBar.StepProgress;
          if ProgressStatusBar.CancelClicked then
            exit;
        end;
      end;
    end
    else if SrcObj is TORTreeView then
    begin
      aTr := TTreeView(SrcObj);
      for I := 0 to aTr.Items.Count - 1 do
      begin
        if aTr.Items[I].Selected then
        begin
          // Check each order to see if we can move this over
          if not assigned(aTr.Items[I].Data) then
            Continue;
          CheckForIssues(aTr.Items[I], nil, ProvObj, PatLst, TmpBool);
          if TmpBool then
            NonXferOrders := true;
        end;
      end;
    end;

    // Check for any records that were found and display any warnings that are needed
    ChkRst := DisplayPatientCheck(PatLst, fViewType, NonXferOrders);
    if ChkRst = mrIgnore  then
    begin
      for I := low(PatLst) to high(PatLst) do
      begin
        if PatLst[I].SelObj is TTreeNode then
          TTreeNode(PatLst[I].SelObj).Selected := false
        else if PatLst[I].SelObj is TListItem then
          TListItem(PatLst[I].SelObj).Selected := false;
        //Clear the order flags
        if (PatLst[I].Reason = txrPend) or (PatLst[I].Reason = txrCancel) then
          fOrderEngine.ClearOrderFlagsByPatient(PatLst[i].IEN);
      end;
    end else if ChkRst = mrCancel then
    begin
     //Clear the selections
     if SrcObj is TListView then
      TListView(SrcObj).ClearSelection
     else if SrcObj is TORTreeView then
      TTreeView(SrcObj).ClearSelection();
     //Clear the order flags
     for I := low(PatLst) to high(PatLst) do
      fOrderEngine.ClearOrderFlagsByPatient(PatLst[i].IEN);
    end;
  Finally
    freeAndNil(fPatDict);
  end;

end;

procedure TFormReassignOrders.ActionApplyChangesExecute(Sender: TObject);
// user wants us to send the assignments to the server for action
begin
  if ReviewAndApplyChanges(fOrderEngine, PrepareReassignments) then
  begin
    fAutoSizedCols := false;
    UpdateScreenAfterReassignment;
  end;
end;

procedure TFormReassignOrders.actionAuditReportExecute(Sender: TObject);
begin
  RunAuditReport;
  fPSB.DisplayHints := true;
end;

procedure TFormReassignOrders.AddNewProviderToList(ANewProvider: tPersonRecord);
var
  AProviderNode: TORTreeNode;
  // aXferRec: pXferToRec;
  XferRecord: tTransferToRec;
begin
  // add this provider to the treelist only if not already in the list
  if (ANewProvider.IEN <> '') and (ProviderExistsInTreeList(ANewProvider.IEN)
    = nil) then
  begin
    AProviderNode := TORTreeNode(tvNewAssignments.Items.Add(nil,
      ANewProvider.ToNodeName));
    AProviderNode.StringData := ANewProvider.ToStringData;
    AProviderNode.Tag := ANewProvider.Tag;
    AProviderNode.Selected := false;

    // XferRecord := TransferOrdersLst.Add(AProviderNode);
    // System.New(aXferRec);
    XferRecord.IsProviderDisplay := true;
    XferRecord.PatientInfo.Assign(ANewProvider);
    fTransferOrdersLst.Add(AProviderNode, XferRecord);
    // AProviderNode.Data := aXferRec;
    SetHeaderSize(AProviderNode);
    AProviderNode.Selected := true;
  end;
end;

procedure TFormReassignOrders.ActionAddNewProviderExecute(Sender: TObject);
var
  ANewProvider: tPersonRecord;
  StartDate: tDate;
  StopDate: tDate;
  abrk: boolean;
begin
  // start/stop dates are meaningless and not used in this case
  // they're just passed so we can reuse the dialog
  StartDate := fOrderEngine.DateStart;
  StopDate := fOrderEngine.DateStop;

  abrk := false;
  if SelectProviderWithDateRange(tptNew, StartDate, StopDate, ANewProvider) then
  begin
    while (not abrk and (ANewProvider.IEN = fOrderEngine.CurrentProvider.IEN)) do
    begin
      abrk := PRTMessage('Invalid Provider', 'Invalid Provider',
                          Format('%s can not be selected as a provider, please choose another provider', [ANewProvider.name]),
                          ['Select New', 'Cancel'], 'Select New', tdlgError, true, 'Details', Format('%s is already selected as the source provider', [ANewProvider.name])) = 1;
      if not abrk then
        SelectProviderWithDateRange(tptNew, StartDate, StopDate, ANewProvider);
    end;
    if abrk then
     exit;

    AddNewProviderToList(ANewProvider);
  end;

end;

// CB
procedure TFormReassignOrders.ActionChooseCurrentProviderExecute
  (Sender: TObject);
// *******************************************************************
// Loads the order based on the criteria
// *******************************************************************
var
  ACurrentProvider: tPersonRecord;
  StartDate: tDate;
  StopDate: tDate;
  rtnCsr: TCursor;
begin
  // Check to see if there are any unapplied reassignments and verify OK to lose them
  if not CheckAndQueryUnsavedWork then
    exit;

  // cleanup GUI and choose a new provider
  StartDate := fOrderEngine.DateStart;
  StopDate := fOrderEngine.DateStop;
  ACurrentProvider := fOrderEngine.CurrentProvider;
  // Open the provider selection screen
  if SelectProviderWithDateRange(tptCurrent, StartDate, StopDate,
    ACurrentProvider) then
  begin
    rtnCsr := screen.Cursor;
    screen.Cursor := crHourGlass;
    try
      fOrderEngine.CurrentProvider := ACurrentProvider;
      fOrderEngine.DateStart := StartDate;
      fOrderEngine.DateStop := StopDate;
      UpdatePatientsLabel;
      ProgressStatusBar.ResetProgressBar;
      // Make the call to get the orders
      fOrderEngine.GetProviderOrdersFromServer(ProgressStatusBar);
      // Update the column widths
      fAutoSizedCols := false;
      // Populate our view(s)
      PopulateAllPatientsLists;
      // Ensure that any pending move items are cleared
      tvNewAssignments.Items.Clear;
    finally
      screen.Cursor := rtnCsr;
    end;
  end;
end;

procedure TFormReassignOrders.ActionCollapseAllExecute(Sender: TObject);
// user activated a context menu item to collaps one of the tree lists
begin
  // This action might not make sense at first reading
  // The idea is in the future we might call this action from either tree list
  if ActionCollapseAll.ActionComponent = popNewAssignmentsFullCollapse then
    tvNewAssignments.FullCollapse;
end;

procedure TFormReassignOrders.ActionExitExecute(Sender: TObject);
// Close the application - the FormCloseQuery method will check for unsaved work
begin
  Close;
end;

procedure TFormReassignOrders.ActionExpandAllExecute(Sender: TObject);
// user activated a context menu item to expand one of the tree lists
begin
  // This action might not make sense at first reading
  // The idea is in the future we might call this action from either tree list
  if ActionExpandAll.ActionComponent = popNewAssignmentsFullExpand then
    tvNewAssignments.FullExpand;
end;

procedure TFormReassignOrders.actionHelpContentsExecute(Sender: TObject);
begin
  inherited;
  // check for network help file
  LoadhelpFile(Application.HelpFile);
  HtmlHelp(0, Application.HelpFile, HH_DISPLAY_TOC, 0);
  // Issue with keyboard in the help file
  // actionHelpContents.ExecuteTarget(sender);
end;

procedure TFormReassignOrders.ActionRPCLogExecute(Sender: TObject);
// Show the RPC Log Window
begin
  ShowBroker;
end;

Function TFormReassignOrders.GetPopUpControl(Sender: TObject): TComponent;
var
  aPopUp: TPopupMenu;
begin
  // Sender = Action -> TMenuItem -> TPopUpMenu -> TOrTreeView
  result := nil;
  if Sender is TAction then
  begin
    if TAction(Sender).ActionComponent is TMenuItem then
    begin
      if TMenuItem(TAction(Sender).ActionComponent).GetParentComponent is TPopupMenu
      then
      begin
        aPopUp := TPopupMenu(TMenuItem(TAction(Sender).ActionComponent)
          .GetParentComponent);
        result := aPopUp.PopupComponent;
      end;
    end;
  end;
end;

procedure TFormReassignOrders.ActionSelectAllExecute(Sender: TObject);
var
  aCtrl: TComponent;
begin
  aCtrl := GetPopUpControl(Sender);
  if assigned(aCtrl) then
    selectAll(aCtrl, true);
end;

procedure TFormReassignOrders.ActionUnselectAllExecute(Sender: TObject);
var
  aPopUp: TPopupMenu;
begin
  // Sender = Action -> TMenuItem -> TPopUpMenu -> TOrTreeView
  if Sender is TAction then
  begin
    if TAction(Sender).ActionComponent is TMenuItem then
    begin
      if TMenuItem(TAction(Sender).ActionComponent).GetParentComponent is TPopupMenu
      then
      begin
        aPopUp := TPopupMenu(TMenuItem(TAction(Sender).ActionComponent)
          .GetParentComponent);
        selectAll(aPopUp.PopupComponent, false);
      end;
    end;
  end;
end;

procedure TFormReassignOrders.selectAll(AView: TObject; Select: Boolean);
// select or deselect all nodes in a tree list
// I'm surprised a SelectAllNodes wasn't built into the component
var
  nodeEnumerator: TTreeNodesEnumerator;
  Nodes: array of TTreeNode;
  aTree: TORTreeView;
  aList: TListView;
begin
  if AView is TORTreeView then
  begin
    aTree := TORTreeView(AView);
    Nodes := [];
    if not Select then
    begin
      aTree.Select(Nodes);
      exit;
    end;

    nodeEnumerator := TTreeNodesEnumerator.Create(aTree.Items);
    try
      while nodeEnumerator.moveNext do
      begin
        // I don't try to optimize - array is reallocated each iteration
        // that's ok - not performance sensitive
        Nodes := Nodes + [nodeEnumerator.Current];
      end;
    finally
      freeAndNil(nodeEnumerator);
    end;
    aTree.Select(Nodes);
  end
  else if AView is TListView then
  begin
    aList := TListView(AView);
    if Select then
      aList.selectAll
    else
      aList.ClearSelection;
  end;
end;

procedure TFormReassignOrders.ActionShowPatientOrdersExecute(Sender: TObject);
// user wants to see a list of orders for selected patient or provider (either tree list)
// fClickedCurrentPatient was assigned in the mouse down event
// ShowPatientDetail will check for a nil node
var
  aCtrl: TComponent;
begin
  if (ActionShowPatientOrders.ActionComponent = popShowCurrentPatientOrders)
  then
  begin
    aCtrl := GetPopUpControl(Sender);
    if assigned(aCtrl) then
      ShowPatientDetails(aCtrl);
  end
  else if (ActionShowPatientOrders.ActionComponent = popShowNewPatientOrders)
  then
    ShowPatientDetails(tvNewAssignments);
end;

procedure TFormReassignOrders.ActionUndoAllExecute(Sender: TObject);
// move all newly assigned patients back to the current patients list box
// might just want to replace this method with a clear all and another call to populate patient list
var
  ANode: TTreeNode;
  DestObj: TObject;
  rtnCsr: TCursor;
begin
  rtnCsr := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    DestObj := nil;
    if ActionViewPat.Checked then
      DestObj := lvCurrentAssignments
    else if ActionViewXfer.Checked then
      DestObj := lvPatient
    else if ActionViewOrd.Checked then
      DestObj := lvPendOut;

    if not assigned(DestObj) then
      exit;

    ProgressStatusBar.ResetProgressBar(tvNewAssignments.Items.Count * 3);
    ANode := tvNewAssignments.Items[0];
    while assigned(ANode) do
    begin
      if not ANode.Selected then
        tvNewAssignments.Select(ANode, [ssCtrl]);

      //Grab the next node since we will be deleting
      ANode := ANode.getNextSibling;

      ProgressStatusBar.ResetProgressBar(tvNewAssignments.SelectionCount * 3);
      MoveItemsBetweenList(DestObj, tvNewAssignments);
      RemoveItemsFromList(tvNewAssignments);
      ProgressStatusBar.HideProgress;


    end;
    ProgressStatusBar.HideProgress;
  finally
    screen.Cursor := rtnCsr;
  end;
end;

procedure TFormReassignOrders.ActionUndoSelectedExecute(Sender: TObject);
// undo one or more new patient assignments in tree list
var
  DestObj: TListView;
  rtnCsr: TCursor;
begin
  rtnCsr := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    // is a provider or patient selected?
    if tvNewAssignments.Selected = nil then
      exit;

    DestObj := nil;
    if ActionViewPat.Checked then
      DestObj := lvCurrentAssignments
    else if ActionViewXfer.Checked then
      DestObj := lvPatient
    else if ActionViewOrd.Checked then
      DestObj := lvPendOut;

    if not assigned(DestObj) then
      exit;

    ProgressStatusBar.ResetProgressBar(tvNewAssignments.SelectionCount * 3);
    MoveItemsBetweenList(DestObj, tvNewAssignments);
    RemoveItemsFromList(tvNewAssignments);
    ProgressStatusBar.HideProgress;
  finally
    screen.Cursor := rtnCsr;
  end;
end;

procedure TFormReassignOrders.ActionsMainUpdate(Action: TBasicAction;
  var Handled: Boolean);

  function SomethingSelected: Boolean;
  begin
    result := false;
    if ActionViewPat.Checked then
      result := lvCurrentAssignments.SelCount > 0
    else if ActionViewXfer.Checked then
      result := lvPatient.SelCount > 0
    else if ActionViewOrd.Checked then
      result := (lvPendNone.SelCount > 0) or (lvPendOut.SelCount > 0) or
        (lvPendIn.SelCount > 0);
  end;

  function IsAProviderSelectedInTreeList: Boolean;
  // we use this test to determine if a provider is selected at right
  begin
    result := false;
    if assigned(tvNewAssignments.Selected) then
      result := tvNewAssignments.Selected.level = 0;
  end;

begin
  // this method is called during event loop idle periods, but does not use 100% cpu
  ActionAddSelected.Enabled := IsAProviderSelectedInTreeList and
    SomethingSelected;
  ActionAddAll.Enabled := (SomethingSelected) and IsAProviderSelectedInTreeList;
  ActionUndoSelected.Enabled := tvNewAssignments.Selected <> nil;
  ActionUndoAll.Enabled := tvNewAssignments.Items.Count > 0;
  ActionApplyChanges.Enabled := IsThereUnsavedWork;
  ActionAutoAllocate.Enabled := (SomethingSelected) and
    (tvNewAssignments.Items.Count > 0);
  ShowPatientCount;
  // I deliberately do not update ActionShowPatientOrders here, because it is context menu dependent
  // it is updated in the each popup menu's onPopup event
end;

procedure TFormReassignOrders.InitializeTimeoutFunctionality;
// initialize the timeout functionality
begin
  InitTimeOut(TimeOutCondition, TimeOutAction);
  GetUserInfo;
  UpdateTimeOutInterval(User.DTIME * 1000); // DTIME * 1000 mSec
end;

procedure TFormReassignOrders.SetHeaderSize(aTreeNode: TTreeNode = nil);

  procedure CheckWidthForItem(aTreeNode: TTreeNode);
  var
    NewWidth: Integer;
    XferRecord: tTransferToRec;
    aOrdRec: tOrderRecord;
  begin
    XferRecord := fTransferOrdersLst.TransferRec[aTreeNode];

    if not XferRecord.IsProviderDisplay then
      NewWidth := HeaderControl1.Canvas.TextWidth
        (XferRecord.PatientInfo.name) + 80
    else
      NewWidth := HeaderControl1.Canvas.TextWidth
        (XferRecord.PatientInfo.name) + 20;

    if HeaderControl1.Sections[0].Width < NewWidth then
      HeaderControl1.Sections[0].Width := NewWidth;

    if not XferRecord.IsProviderDisplay then
    begin
      if XferRecord.Group_Type = tvtOrder then
      begin
        aOrdRec := fOrderEngine.OrderByIEN(XferRecord.OrderIEN);

        NewWidth := HeaderControl1.Canvas.TextWidth(XferRecord.OrderIEN) + 20;
        if HeaderControl1.Sections[1].Width < NewWidth then
          HeaderControl1.Sections[1].Width := NewWidth;

        NewWidth := HeaderControl1.Canvas.TextWidth(aOrdRec.orderDate) + 20;
        if HeaderControl1.Sections[2].Width < NewWidth then
          HeaderControl1.Sections[2].Width := NewWidth;

        NewWidth := HeaderControl1.Canvas.TextWidth(aOrdRec.orderStatus) + 20;
        if HeaderControl1.Sections[3].Width < NewWidth then
          HeaderControl1.Sections[3].Width := NewWidth;

      end
      else
      begin
        NewWidth := HeaderControl1.Canvas.TextWidth
          (IntToStr(XferRecord.NumOfOrders)) + 20;
        if HeaderControl1.Sections[1].Width < NewWidth then
          HeaderControl1.Sections[1].Width := NewWidth;

      end;
    end;
  end;

var
  I, NewWidth: Integer;
begin
  // SendMessage(Panel3.Handle, WM_SETREDRAW, WPARAM(false), 0);
  try

    for I := 0 to HeaderControl1.Sections.Count - 1 do
    begin
      NewWidth := HeaderControl1.Canvas.TextWidth
        (HeaderControl1.Sections[I].Text) + 20;
      if HeaderControl1.Sections[I].Width < NewWidth then
        HeaderControl1.Sections[I].Width := NewWidth;
    end;
    if assigned(aTreeNode) then
    begin
      CheckWidthForItem(aTreeNode);
    end
    else
    begin
      // Ensure all sections are visible
      for I := 0 to tvNewAssignments.Items.Count - 1 do
      begin
        CheckWidthForItem(tvNewAssignments.Items[I]);
      end;
    end;
    //Update the panel
    NewWidth := 0;
    for I := 0 to HeaderControl1.Sections.Count - 1 do
      Inc(NewWidth, HeaderControl1.Sections[I].Width + 10);
    if Panel3.Width <> NewWidth then
    begin
      Panel3.Width := NewWidth;
      if Panel3.Width < ScrollBox1.Width then
        Panel3.Width := ScrollBox1.Width - 5;
      Panel3.Realign;
    end;
  finally
    // Make sure updates are re-enabled
    // SendMessage(Panel3.Handle, WM_SETREDRAW, WPARAM(true), 0);
    // Invalidate;  // Might be required to reflect the changes
  end;
end;

procedure TFormReassignOrders.HeaderControl1SectionTrack(HeaderControl
  : THeaderControl; Section: THeaderSection; Width: Integer;
  State: TSectionTrackState);
begin
  inherited;
  if State = tsTrackMove then
    tvNewAssignments.Invalidate;
end;

function TFormReassignOrders.ProviderExistsInTreeList(AProviderIEN: string)
  : TTreeNode;
// given a provider name, determine if that provider is already present in the new providers tree list
var
  nodeEnumerator: TTreeNodesEnumerator;
begin
  result := nil;
  nodeEnumerator := TTreeNodesEnumerator.Create(tvNewAssignments.Items);
  try
    while nodeEnumerator.moveNext do
      if NodeToPerson(TORTreeNode(nodeEnumerator.Current)).IEN = AProviderIEN
      then
      begin
        result := nodeEnumerator.Current;
        break;
      end;
  finally
    freeAndNil(nodeEnumerator);
  end;
end;

{function TFormReassignOrders.GatherAllNewProviders: TTreeNodeArray;
// given a provider name, determine if that provider is already present in the new providers tree list
var
  nodeEnumerator: TTreeNodesEnumerator;
begin
  result := nil;
  nodeEnumerator := TTreeNodesEnumerator.Create(tvNewAssignments.Items);
  try
    while nodeEnumerator.moveNext do
      if TORTreeNode(nodeEnumerator.Current).level = 0 then
        result := result + [nodeEnumerator.Current];
  finally
    freeAndNil(nodeEnumerator);
  end;
end;
}
procedure TFormReassignOrders.ScrollBox1Resize(Sender: TObject);
var
  NewWidth, I: integer;
begin
  inherited;
  if Panel3.Height <> ScrollBox1.Height then
    Panel3.Height := ScrollBox1.Height;
  if Panel3.Width > ScrollBox1.Width then
  begin
    NewWidth := 0;
    for I := 0 to HeaderControl1.Sections.Count - 1 do
      Inc(NewWidth, HeaderControl1.Sections[I].Width + 10);
    if Panel3.Width <> NewWidth then
      Panel3.Width := NewWidth;
  end else if Panel3.Width < ScrollBox1.Width then
    Panel3.Width := ScrollBox1.Width - 5;
  Panel3.Realign;
end;

procedure TFormReassignOrders.RemoveItemsFromList(SrcObj: TObject);
var
  aLV: TListView;
  aORT: TORTreeView;
  I: Integer;
begin
  if SrcObj is TListView then
  begin
    aLV := TListView(SrcObj);
    aLV.Items.BeginUpdate;
    try
      // Need to clean removed items from the list
      ProgressStatusBar.ResetLoop(aLV.SelCount);
      for I := aLV.Items.Count - 1 downto 0 do
      begin
        if aLV.Items[I].Selected then
        begin
          if (aLV = lvPendNone) then
          begin
            fOrderEngine.OrdersPendingNone.Delete(I);
            lvPendNone.Items.Count := lvPendNone.Items.Count - 1;
          end
          else if (aLV = lvPendOut) then
          begin
            fOrderEngine.OrdersPendingOut.Delete(I);
            lvPendOut.Items.Count := lvPendOut.Items.Count - 1;
          end
          else if (aLV = lvPendIn) then
          begin
            fOrderEngine.OrdersPendingIn.Delete(I);
            lvPendIn.Items.Count := lvPendIn.Items.Count - 1;
          end
          else
          begin
            if not assigned(aLV.Items[I].Data) then
              Continue;
            aLV.Items[I].Delete;
          end;
          ProgressStatusBar.StepProgress;
          if ProgressStatusBar.CancelClicked then
            exit;
        end;
      end;
    finally
      aLV.Items.EndUpdate;
      aLV.Invalidate;
    end;

  end
  else if SrcObj is TORTreeView then
  begin
    aORT := TORTreeView(SrcObj);
    aORT.Items.BeginUpdate;
    try
      fTreeView_Deleting := true;

      if tvNewAssignments.Items.Count = 0 then
        exit;


      while aORT.SelectionCount > 0 do
     // for I := aORT.SelectionCount - 1 downto 0 do
      begin
        if aORT.Selections[0].hasChildren then
          aORT.Selections[0].DeleteChildren;
        aORT.Selections[0].Delete;
      end;

      fTreeView_Deleting := false;
    finally
      aORT.Items.EndUpdate;
    end;

  end
end;

procedure TFormReassignOrders.ShowPatientCount;
// display a count beneath the current patients list box
var
  Total: Integer;
begin
  case fViewType of
    tvtPatient:
      lblPatientCount.Caption := format('%d current patients',
        [lvCurrentAssignments.Items.Count]);
    tvtXfer:
      lblPatientCount.Caption := format('%d current patients',
        [fOrderEngine.TotalPatients]);
    tvtOrder:
      begin
        Total := lvPendNone.Items.Count + lvPendOut.Items.Count +
          lvPendIn.Items.Count;
        lblPatientCount.Caption := format('%d orders', [Total]);
      end;
  end;

end;

Procedure TFormReassignOrders.MoveItemsBetweenList(DestLst, SrcObj: TObject;
  AParentNode: TTreeNode = nil);
 var
  LenNA, LenPO, LenPI: Integer;

  Procedure AddToGroupedList(aXferRec: tTransferToRec);
  var
    aListObj: pListObj;
    aLvItm: TListItem;
    aOrderRec: tOrderRecord;
    aGroup: TListGroup;
    TmpGrpTxt: String;
    tmpGrpID: Integer;
  begin
    aGroup := nil;
    System.New(aListObj);
    // since this is all the orders for this patient then we only need the first
    // order for the lookup
    aListObj.Group_Type := aXferRec.Group_Type;
    aListObj.patientIEN := aXferRec.PatientInfo.IEN;
    aListObj.OrderIEN := aXferRec.OrderIEN;
    aListObj.NumOfOrders := aXferRec.NumOfOrders;
    // aListObj.Assign(aXferRec.ListObject);
    aOrderRec := fOrderEngine.OrderByIEN(aXferRec.OrderIEN);

    if aXferRec.Group_Type = tvtXfer then
    begin
      TListView(DestLst).GroupView := true;

      case aOrderRec.OrderXfer of
        txftPendOut:
          begin
            tmpGrpID := grp_out;
            TmpGrpTxt := grp_out_Header;
          end;
        txftPendIn:
          begin
            tmpGrpID := grp_In;
            TmpGrpTxt := grp_in_Header;
          end;
        else begin
          tmpGrpID := grp_None;
          TmpGrpTxt := grp_none_Header;
        end;
      end;

      aGroup := TListView(DestLst).Groups.FindItemByGroupID(tmpGrpID);
      if not assigned(aGroup) then
      begin
        aGroup := TListView(DestLst).Groups.Add;
        aGroup.GroupID := tmpGrpID;
        aGroup.Header := TmpGrpTxt;
      end;

    end;

    aLvItm := TListView(DestLst).Items.Add;
    if aXferRec.Group_Type = tvtXfer then
      aLvItm.GroupID := aGroup.GroupID;
    aLvItm.Caption := aOrderRec.patient.name;
    aLvItm.SubItems.Add(IntToStr(aXferRec.NumOfOrders));
    aLvItm.Data := aListObj;
  end;

  Procedure AddOrdersBack(XferRecord: tTransferToRec);
  var
   aOrdRec: tOrderRecord;
  begin
    case XferRecord.Group_Type of
          tvtPatient:
          begin
           AddToGroupedList(XferRecord);
           if XferRecord.OrderFlag <> tofNA then
            fOrderEngine.ClearOrderFlagsByPatient(XferRecord.PatientInfo.IEN);
          end;
          tvtXfer:
          begin
            AddToGroupedList(XferRecord);
            if XferRecord.OrderFlag <> tofNA then
            begin
              aOrdRec := fOrderEngine.OrderByIEN(XferRecord.OrderIEN);
              fOrderEngine.ClearOrderFlagsByPatientAndXfer(XferRecord.PatientInfo.IEN, aOrdRec.OrderXfer);
            end;
          end;
          tvtOrder:
            begin
              aOrdRec := fOrderEngine.OrderByIEN(XferRecord.OrderIEN);
              if aOrdRec.OrderFlag <> tofNA then
                fOrderEngine.ClearOrderFlagsByOrder(XferRecord.OrderIEN);
              case aOrdRec.OrderXfer of
               { xfer_None:
                  fOrderEngine.OrdersPendingNone.Add(aOrdRec);
                xfer_PendOut:
                  fOrderEngine.OrdersPendingOut.Add(aOrdRec);
                xfer_PendIn:
                  fOrderEngine.OrdersPendingIn.Add(aOrdRec);   }
                txftPendOut:
                begin
                   Inc(LenPO);
                     fOrderEngine.OrdersPendingOut[LenPO] := aOrdRec;
                end;
                txftPendIn:
                begin
                   Inc(LenPI);
                     fOrderEngine.OrdersPendingIn[LenPI] := aOrdRec;
                end;
                else begin
                  Inc(LenNA);
                  fOrderEngine.OrdersPendingNone[LenNA] := aOrdRec;
                end;
              end;
            end;
        end;
        ProgressStatusBar.StepProgress;
        if ProgressStatusBar.CancelClicked then
          exit;
  end;

var
  aTree, bTree: TORTreeView;
  aLV: TListView;
  I, X, ProgCnt: Integer;
  plstObj: pListObj;
  aOrdRec: tOrderRecord;
  NewNode: TORTreeNode;
  ANode: TTreeNode;
  NodeString: String;
  XferRecord: tTransferToRec;
  nodesArray: TArray<TTreeNode>;
begin
  plstObj := nil;
  if (DestLst is TORTreeView) then
  begin
    if not assigned(AParentNode) then
      exit;

    if AParentNode.level > 0 then
      AParentNode := AParentNode.parent;

    CanMoveToProvider(SrcObj, AParentNode);
    aTree := TORTreeView(DestLst);
    aTree.Items.BeginUpdate;
    try
      if (SrcObj is TListView) then
      begin
        aLV := TListView(SrcObj);
        ProgressStatusBar.ResetLoop(aLV.SelCount);
        for I := 0 to aLV.Items.Count - 1 do
        begin
          if aLV.Items[I].Selected then
          begin
            if (aLV = lvPendNone) or (aLV = lvPendOut) or (aLV = lvPendIn) then
            begin
              if aLV = lvPendNone then
                aOrdRec := fOrderEngine.OrdersPendingNone[aLV.Items[I].Index]
              else if aLV = lvPendOut then
                aOrdRec := fOrderEngine.OrdersPendingOut[aLV.Items[I].Index]
              else if aLV = lvPendIn then
                aOrdRec := fOrderEngine.OrdersPendingIn[aLV.Items[I].Index];
            end
            else
            begin
              if not assigned(aLV.Items[I].Data) then
                Continue;
              plstObj := aLV.Items[I].Data;
              aOrdRec := fOrderEngine.OrderByIEN(plstObj.OrderIEN);
            end;

            if not aOrdRec.patient.IsProvider then
            begin
              NewNode := TORTreeNode(aTree.Items.AddChild(AParentNode,
                aOrdRec.patient.ToNodeName));

              NodeString := U + aOrdRec.patient.name;
              if (aLV = lvPendNone) or (aLV = lvPendOut) or (aLV = lvPendIn) then
                NodeString := NodeString + U + aOrdRec.OrderIEN
              else
                NodeString := NodeString + U + IntToStr(tListObject(plstObj^).NumOfOrders);
              NodeString := NodeString + U + aOrdRec.orderDate;
              NodeString := NodeString + U + aOrdRec.orderStatus;

              NewNode.StringData := aOrdRec.patient.ToStringData + NodeString;
              NewNode.Tag := aOrdRec.patient.Tag;

              XferRecord.IsProviderDisplay := false;

              XferRecord.OrderIEN := aOrdRec.OrderIEN;
              XferRecord.OrderFlag := aOrdRec.orderFlag;
              if (aLV = lvPendNone) or (aLV = lvPendOut) or (aLV = lvPendIn)
              then
                XferRecord.Group_Type := tvtOrder
              else
              begin
                if not assigned(plstObj) then
                  exit;
                XferRecord.Group_Type := tListObject(plstObj^).Group_Type;
                XferRecord.NumOfOrders := tListObject(plstObj^).NumOfOrders;
              end;

              XferRecord.PatientInfo.Assign(aOrdRec.patient);
              XferRecord.PatientInfo.OrderXfer := aOrdRec.OrderXfer;


              XferRecord.TransferTo.Assign
                (NodeToPerson(TORTreeNode(AParentNode)));
              fTransferOrdersLst.Add(NewNode, XferRecord);

              SetHeaderSize(NewNode);

              ProgressStatusBar.StepProgress;
              if ProgressStatusBar.CancelClicked then
                exit;

            end;

          end;
        end;
      end
      else
      begin
        bTree := TORTreeView(SrcObj);

        setLength(nodesArray, bTree.SelectionCount);
        for I := 0 to high(nodesArray) do
          nodesArray[I] := bTree.Selections[I];
        ProgressStatusBar.ResetLoop(bTree.SelectionCount);
        for ANode in nodesArray do
        begin
          ANode.MoveTo(AParentNode, naAddChild);
          ANode.Selected := false;
          ProgressStatusBar.StepProgress;
          if ProgressStatusBar.CancelClicked then
            exit;
        end;

      end;
    finally
      aTree.Items.EndUpdate;
    end;
    aTree.FullExpand;

  end
  else if (DestLst is TListView) and (SrcObj is TORTreeView) then
  begin
    aLV := TListView(DestLst);
    bTree := TORTreeView(SrcObj);
    if fViewType <> tvtOrder then
      aLV.Items.BeginUpdate
    else begin
      lvPendNone.Items.BeginUpdate;
    end;
    try
      // Highlight the children, if needed
      if tvNewAssignments.Items.Count = 0 then
        exit;

      ProgCnt := 0;
      for I := 0 to bTree.SelectionCount - 1 do
      begin
        if not bTree.Selections[I].hasChildren then
         Inc(ProgCnt)
        Else begin
          for X := bTree.Selections[I].Count - 1 downto 0 do
            Inc(ProgCnt);
        end;
      end;

      ProgressStatusBar.ResetLoop(ProgCnt);
      ProgressStatusBar.ResetProgressBar(ProgCnt);

      //Preset the array
      if fViewType = tvtOrder then
      begin
        LenNA := High(fOrderEngine.OrdersPendingNone);
        LenPO := High(fOrderEngine.OrdersPendingOut);
        LenPI := High(fOrderEngine.OrdersPendingIn);
        fOrderEngine.OrdersPendingNone.SetSize(Length(fOrderEngine.OrdersPendingNone) + ProgCnt);
        fOrderEngine.OrdersPendingOut.SetSize(Length(fOrderEngine.OrdersPendingOut) + ProgCnt);
        fOrderEngine.OrdersPendingIn.SetSize(Length(fOrderEngine.OrdersPendingIn) + ProgCnt);
      end;

      for I := 0 to bTree.SelectionCount - 1 do
      begin
        //If the parent is selcetd then we will get the children when we hit the parent
        if Assigned(bTree.Selections[I].Parent) and bTree.Selections[I].Parent.Selected then
        begin
            Continue;
        end else begin
          if not bTree.Selections[I].hasChildren then
           AddOrdersBack(fTransferOrdersLst.TransferRec[bTree.Selections[I]])
          else begin
            for X := bTree.Selections[I].Count - 1 downto 0 do
              AddOrdersBack(fTransferOrdersLst.TransferRec[bTree.Selections[I].Item[X]])
          end;
        end;
      end;

      //Trim the array
      if fViewType = tvtOrder then
      begin
        fOrderEngine.OrdersPendingNone.SetSize(LenNA+1);
        fOrderEngine.OrdersPendingOut.SetSize(LenPO+1);
        fOrderEngine.OrdersPendingIn.SetSize(LenPI+1);
      end;

      if lvPendNone.Items.Count <> length(fOrderEngine.OrdersPendingNone) then
      begin
        fOrderEngine.OrdersPendingNone.SortByName;
        lvPendNone.Items.Count := length(fOrderEngine.OrdersPendingNone);
        lvPendNone.Invalidate;
      end;
      if lvPendOut.Items.Count <> length(fOrderEngine.OrdersPendingOut) then
      begin
        fOrderEngine.OrdersPendingOut.SortByName;
        lvPendOut.Items.Count := length(fOrderEngine.OrdersPendingOut);
        lvPendOut.Invalidate;
      end;
      if lvPendIn.Items.Count <> length(fOrderEngine.OrdersPendingIn) then
      begin
        fOrderEngine.OrdersPendingIn.SortByName;
        lvPendIn.Items.Count := length(fOrderEngine.OrdersPendingIn);
        lvPendIn.Invalidate;
      end;

    finally
      if fViewType <> tvtOrder then
      begin
        aLV.AlphaSort;
        aLV.Items.EndUpdate;
      end else begin
        lvPendNone.Items.EndUpdate;
      end;
    end;
  end;
end;

function TFormReassignOrders.NodeToPerson(ANode: TORTreeNode): tPersonRecord;
// given a tree node, extract the IEN and Name into a record
begin
  result.IEN := Piece(ANode.StringData, U, ID_IEN);
  result.name := Piece(ANode.StringData, U, ID_NAME);
  result.Tag := ANode.Tag;
  result.IsProvider := StrToBool(Piece(ANode.StringData, U, ID_PROVIDER));
end;

procedure TFormReassignOrders.pmCurrentPopup(Sender: TObject);
begin
  if Sender is TPopupMenu then
  begin
    if TPopupMenu(Sender).PopupComponent is TListView then
    begin
      ActionShowPatientOrders.Enabled :=
        TListView(TPopupMenu(Sender).PopupComponent).SelCount > 0;
      ActionSelectAll.Enabled := TListView(TPopupMenu(Sender).PopupComponent)
        .Items.Count > 0;
      ActionUnselectAll.Enabled := TListView(TPopupMenu(Sender).PopupComponent)
        .SelCount > 0;
    end;
  end;
end;

procedure TFormReassignOrders.pmNewPopup(Sender: TObject);
// see my comment in the actions.OnUpdate event
// since the action is shared with two components, I make this action update menu specific
begin
  ActionShowPatientOrders.Enabled := tvNewAssignments.SelectionCount > 0;
  if Sender is TPopupMenu then
  begin
    if TPopupMenu(Sender).PopupComponent is TTreeView then
    begin
      ActionSelectAll.Enabled := TTreeView(TPopupMenu(Sender).PopupComponent)
        .Items.Count > 0;
      ActionUnselectAll.Enabled := TTreeView(TPopupMenu(Sender).PopupComponent)
        .SelectionCount > 0;
    end;
  end;
end;

procedure TFormReassignOrders.pnlRightResize(Sender: TObject);
begin
  // inherited;
  if assigned(ProgressStatusBar) then
    ProgressStatusBar.AdjustPanelWidth(0, pnlRight.Width * -1);
end;

function TFormReassignOrders.PendingChangesDlg: Integer;
var
  Cap, Title, Body, TechTitle, TechBody: String;
begin
  Cap := 'Confirm Discard';
  Title := 'Discard Changes';
  Body := 'Are you sure you want to cancel all pending transfer assignments?';
  TechTitle := 'Technical information';
  TechBody := 'Click the Keep button to return to the transfer screen. If you discard the pending transfer assignments, then you will have to reenter the transfers';
  Result := PRTMessage(cap, Title, Body, ['Discard', 'Keep'], 'Keep', tdlgWarning, true, TechTitle, TechBody);
end;

function TFormReassignOrders.CheckAndQueryUnsavedWork: Boolean;
// ******************************************************************************
// return true if there is no unsaved work, or the user says it's OK to
// discard without applying
// ******************************************************************************
begin
  // Check for work
  result := not IsThereUnsavedWork;
  if not result then
  begin
    result := PendingChangesDlg = 0;
  end;
end;

procedure TFormReassignOrders.UpdateScreenAfterReassignment;
// we've just done a reassignment for better or worse
// results are already in the activity log
// the user has been notified of results
// now we wipe the right-side assignments tree list clean, and we refill the left side patients tree list
begin
  tvNewAssignments.Items.Clear;
  // todo - next two lines stolen - should I refactor into a method?
  fOrderEngine.GetProviderOrdersFromServer(ProgressStatusBar);
  PopulateAllPatientsLists;
end;

// CB
procedure TFormReassignOrders.PopulateAllPatientsLists;
// ******************************************************************************
// Display the orders depending on the tool menu
// ******************************************************************************
begin
  // Show total patient count if not in specific order view
  lblPatientCount.visible := not ActionViewOrd.Checked;

  if ActionViewPat.Checked then
    PopulatePatientList(fOrderEngine.Orders)
  else if ActionViewXfer.Checked then
    PopulateXferList(fOrderEngine.Orders)
  else if ActionViewOrd.Checked then
    PopulateOrderList(fOrderEngine.Orders);
  ProgressStatusBar.HideProgress;
end;

// CB
Procedure TFormReassignOrders.PopulatePatientList(const Orders: tOrderRecords);
// ******************************************************************************
// Populates the pateint view
// ******************************************************************************
var
  order: tOrderRecord;
  alvItem: TListItem;
  rtnCursor: TCursor;
begin
  // Setup the cursor
  rtnCursor := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    // ensure the ListView and Dictionary are clear
    lvCurrentAssignments.Clear;
    fLVDict.Clear;
    // Begin the view update
    lvCurrentAssignments.Items.BeginUpdate;
    try
      // Set the column widths to zero to speed up the load
      lvCurrentAssignments.Columns[0].Width := 0;
      lvCurrentAssignments.Columns[1].Width := 0;
      // Pre allocate the list
      lvPatient.AllocBy := High(Orders);
      // Reset the progress bar and the max
      ProgressStatusBar.ResetProgressBar(length(fOrderEngine.Orders));
      ProgressStatusBar.ResetLoop(length(Orders));
      // Loop through each order from our list and add them
      for order in Orders do
      begin
        // Have we already added this patient?
        alvItem := lvCurrentAssignments.FindListItemByPatient
          (order.patient.IEN, fLVDict);
        // If we have already added this patient then we need to increaser the order count
        if assigned(alvItem) then
          lvCurrentAssignments.IncOrderCount(alvItem)
        else
          lvCurrentAssignments.AddListItemGroupByPatient(order, fLVDict);
        // Step the progress and max
        ProgressStatusBar.StepProgress;
        // If canceled clicked then exit
        if ProgressStatusBar.CancelClicked then
          break;
      end;
      // Reset the column widths
      if length(Orders) > 0 then
        lvCurrentAssignments.Columns[0].Width := -1
      else
        lvCurrentAssignments.Columns[0].Width := -2;
      lvCurrentAssignments.Columns[1].Width := -2;
      // Clean up the dictionary
      fLVDict.TrimExcess;
      // Sort the list
      lvCurrentAssignments.AlphaSort;
    finally
      // Finish the update
      lvCurrentAssignments.Items.EndUpdate;
    end;
  finally
    // Retrun the screen cursor
    screen.Cursor := rtnCursor;
  end;
end;

// CB
Procedure TFormReassignOrders.PopulateXferList(const Orders: tOrderRecords);
// ******************************************************************************
// Populates the pateint view by transfer state (grouped)
// ******************************************************************************
var
  order: tOrderRecord;
  alvItem: TListItem;
  rtnCursor: TCursor;
begin
  // Sort the orders list by transfer to speed things up
  fOrderEngine.SortByXfer;
  // Setup the cursor
  rtnCursor := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    // ensure the ListView and Dictionary are clear
    lvPatient.Clear;
    fLVDict.Clear;
    // begin the update
    lvPatient.Items.BeginUpdate;
    try
      // Pre allocate the list
      lvPatient.AllocBy := High(Orders);
      // Set the column widths to zero to speed up the load
      lvPatient.Columns[0].Width := 0;
      lvPatient.Columns[1].Width := 0;
      // Reset the progress bar and the max
      ProgressStatusBar.ResetProgressBar(length(fOrderEngine.Orders));
      ProgressStatusBar.ResetLoop(length(Orders));
      // Loop through each order from our list and add them
      for order in Orders do
      begin
        // Add items but group by the transfer type (same listview)
        alvItem := lvPatient.FindListItemByXfer(order.patient.IEN,
          order.OrderXfer, fLVDict);
        if assigned(alvItem) then
          lvPatient.IncOrderCount(alvItem)
        else
          lvPatient.AddListItemGroupByXfer(order, fLVDict);
        // Step the progress and max
        ProgressStatusBar.StepProgress;
        // If canceled clicked then exit
        if ProgressStatusBar.CancelClicked then
          break;
      end;
      // Clean up the dictionary
      fLVDict.TrimExcess;
      // Reset the column widths
      if length(Orders) > 0 then
        lvPatient.Columns[0].Width := -1
      else
        lvPatient.Columns[0].Width := -2;
      lvPatient.Columns[1].Width := -2;
    Finally
      // Finish the update
      lvPatient.Items.EndUpdate;
    end;
  finally
    // Retrun the screen cursor
    screen.Cursor := rtnCursor;
  end;
end;

// CB
procedure TFormReassignOrders.SetColumnWidth();
// ******************************************************************************
// Resize the virtual list views based on the new data
// ******************************************************************************

  procedure AutoResizeColumn(const AListView: TListView; const AColumn: Integer;
    Const ListSrc: tOrderRecords);
  // ******************************************************************************
  // Auto resize a column
  // ******************************************************************************
  var
    S: string;
    I: Integer;
    MaxWidth: Integer;
    ItemWidth: Integer;
  begin
    // Grab the widht of the columns caption
    MaxWidth := ListView_GetStringWidth(AListView.Handle,
      PChar(AListView.Columns.Items[AColumn].Caption));
    // Grab the width of the text that should display in that column
    for I := low(ListSrc) to High(ListSrc) do
    begin
      // Waht text are we looking for?
      case AColumn of
        0:
          S := ListSrc[I].patient.name;
        1:
          S := ListSrc[I].OrderIEN;
        2:
          S := ListSrc[I].orderDate;
        3:
          S := ListSrc[I].orderStatus;
        4:
          S := ListSrc[I].orderXferEffective;
        5:
          S := ListSrc[I].orderXferFrom.name;
      end;
      // Get the text width
      ItemWidth := ListView_GetStringWidth(AListView.Handle, PChar(S));
      // Update our width if needed
      if MaxWidth < ItemWidth then
        MaxWidth := ItemWidth;
    end;
    // Pad the column
    if AColumn = 0 then
      MaxWidth := MaxWidth + 7
    else
      MaxWidth := MaxWidth + 14;
    // Set the column width
    AListView.Columns.Items[AColumn].Width := MaxWidth;
  end;

var
  I: Integer;
begin
  // Loop through all the columns in the virtual list view
  for I := 0 to lvPendNone.Columns.Count - 1 do
    AutoResizeColumn(lvPendNone, I, fOrderEngine.OrdersPendingNone);

  for I := 0 to lvPendOut.Columns.Count - 1 do
    AutoResizeColumn(lvPendOut, I, fOrderEngine.OrdersPendingOut);

  for I := 0 to lvPendIn.Columns.Count - 1 do
    AutoResizeColumn(lvPendIn, I, fOrderEngine.OrdersPendingIn);

end;

// CB
Procedure TFormReassignOrders.PopulateOrderList(const Orders: tOrderRecords);
// ******************************************************************************
// Populates the order view
// ******************************************************************************
var
  rtnCursor: TCursor;
begin
  //If not auto sized then adjust
  if not fAutoSizedCols then
  begin
    // Setup the cursor
    rtnCursor := screen.Cursor;
    screen.Cursor := crHourGlass;
    ProgressStatusBar.ReturnCursor := crHourGlass;
    try
      // Reset the list
      fOrderEngine.SetupOrdersBySeperateXfer;
      // Set the virtual list count
      lvPendNone.Items.Count := length(fOrderEngine.OrdersPendingNone);
      lvPendOut.Items.Count := length(fOrderEngine.OrdersPendingOut);
      lvPendIn.Items.Count := length(fOrderEngine.OrdersPendingIn);
      // Update the labels
      lblOutTotal.Caption := IntToStr(fOrderEngine.TotalPatientsXferOut) +
        ' Patients';
      lblInTotal.Caption := IntToStr(fOrderEngine.TotalPatientsXferIn) +
        ' Patients';
      lblNoneTotal.Caption := IntToStr(fOrderEngine.TotalPatientsXferNone) +
        ' Patients';
      //resize columns based on data (owner data)
      SetColumnWidth;
      //No need to run again until reload of orders
      fAutoSizedCols := true;
    finally
      // Retrun the screen cursor
      screen.Cursor := rtnCursor;
    end;
  end;
end;

// CB
procedure TFormReassignOrders.ShowPatientDetails(AHostObj: TObject;
  const aFromIEN: string = '');
// *****************************************************************************
// show the order details dialog for the selected patient(s) based on view
// *****************************************************************************
begin
  ShowPatientOrders(fOrderEngine, AHostObj, aFromIEN);
end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsAdvancedCustomDrawItem
  (Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
// *****************************************************************************
// Handle the view of the tree veiw with the added columns
// *****************************************************************************
const
  AlignmentFlags: array [TAlignment] of ULONG = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  LastSection, Section: THeaderSection;
  LineRect, r: TRect;
  clBackground, clText: TColor;
  I: Integer;
  Flags: ULONG;
  tmpRec: tTransferToRec;
  aOrdRec: tOrderRecord;
  OutText: String;
begin
  // Setup the defaults
  PaintImages := true;
  DefaultDraw := true;
  // If we are painting
  if Stage = cdPostPaint then
  begin
    // Grab the last header section
    LastSection := HeaderControl1.Sections[HeaderControl1.Sections.Count - 1];
    // Get the current line rectangle
    LineRect := Node.DisplayRect(true);
    LineRect.Right := LastSection.Left + LastSection.Width;
    // If selected then this needs to be highlighted
    if cdsSelected in State then
    begin
      clBackground := clHighlight;
      clText := clHighlightText;
    end
    else
    begin
      clBackground := tvNewAssignments.Color;
      clText := tvNewAssignments.Font.Color;
    end;
    // Fill out the nodes rectangle
    Sender.Canvas.Brush.Color := clBackground;
    Sender.Canvas.Font.Color := clText;
    Sender.Canvas.FillRect(LineRect);

    r := LineRect;
    Flags := DT_SINGLELINE or DT_NOPREFIX or DT_END_ELLIPSIS or DT_VCENTER;
    // Ensure that we have some data to pull from (xfer record)
    // if not assigned(Node.Data) then
    // exit;
    // Load up the xfer record
    tmpRec.NodePointer := -1;
    tmpRec := fTransferOrdersLst.TransferRec[Node];
    if tmpRec.NodePointer = -1 then
      exit;
    // TransferOrdersLst.TransferRecord(Node, tmpRec);
    // Loop through our "columns"
    for I := 0 to HeaderControl1.Sections.Count - 1 do
    begin
      // grab the current secton
      OutText := '';
      Section := HeaderControl1.Sections[I];
      // Align the text
      if I <> 0 then
        r.Left := Section.Left;
      r.Right := r.Left + Section.Width;
      if Section.Alignment in [taLeftJustify, taCenter] then
        inc(r.Left, 2);
      if Section.Alignment in [taRightJustify, taCenter] then
        Dec(r.Right, 2);
      // If we are showing the provider display the name
      if tmpRec.IsProviderDisplay then
      begin
        case I of
          0:
            OutText := tmpRec.PatientInfo.name;
        end;
      end
      else
      begin
        // If we are showing individual orders then display the info
        if tmpRec.Group_Type = tvtOrder then
        begin
          aOrdRec := fOrderEngine.OrderByIEN(tmpRec.OrderIEN);
          case I of
            0:
              OutText := tmpRec.PatientInfo.name;
            1:
              OutText := tmpRec.OrderIEN;
            2:
              OutText := aOrdRec.orderDate;
            3:
              OutText := aOrdRec.orderStatus;
          end;
        end
        else
        begin
          // Showing grouped orders so show the patient and number of orders
          case I of
            0:
              OutText := tmpRec.PatientInfo.name;
            1:
              OutText := IntToStr(tmpRec.NumOfOrders);
          end;
        end;
      end;
      // Draw the actual text
      // with pXferToRec(Node.Data)^ do
      DrawText(Sender.Canvas.Handle, PChar(OutText), length(OutText), r,
        Flags or AlignmentFlags[Section.Alignment]);
    end;
    // Make sure we set focused
    if cdsFocused in State then
      Sender.Canvas.DrawFocusRect(LineRect);
  end;
end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsChanging(Sender: TObject;
  Node: TTreeNode; var AllowChange: Boolean);
// *****************************************************************************
// If we are deleting then do not allow the change
// *****************************************************************************
begin
  inherited;
  if fTreeView_Deleting then
    AllowChange := false;
end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsDblClick(Sender: TObject);
// *****************************************************************************
// show orders window for double-clicked current patient
// *****************************************************************************
begin
  // Grab the first selected order when double clicking
  ShowPatientDetails(tvNewAssignments);
end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsDeletion(Sender: TObject;
  Node: TTreeNode);
// *****************************************************************************
// Ensure we clean up any linked objects when nodes are removed
// *****************************************************************************
begin
  inherited;
  fTransferOrdersLst.Delete(Node);
  // if assigned(Node.Data) then
  // System.Dispose(pXferToRec(Node.Data));
end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsDragDrop(Sender, Source: TObject;
  X, Y: Integer);
// *****************************************************************************
// patients have been dragged and dropped onto the tree list
// *****************************************************************************
var
  nodeNewProvider: TTreeNode;
  aLvObj: TObject;
  rtnCursor: TCursor;
begin
  // Setup the cursor
  rtnCursor := screen.Cursor;
  screen.Cursor := crHourGlass;
  try
    // obtain the potential new provider (parent if needed)
    nodeNewProvider := tvNewAssignments.GetNodeAt(X, Y);
    if assigned(nodeNewProvider) then
    begin
      if nodeNewProvider.level > 0 then
        nodeNewProvider := nodeNewProvider.parent;
    end;
    // Default
    aLvObj := nil;
    // Figure out the source
    if Source is TORTreeView then
    begin
      // If moving with in set the source to iteself
      aLvObj := tvNewAssignments;
    end
    else
    begin
      // Set the source based on the view
      if ActionViewPat.Checked then
        aLvObj := lvCurrentAssignments
      else if ActionViewXfer.Checked then
        aLvObj := lvPatient
      else if ActionViewOrd.Checked then
      begin
        // which virtual listy has a selection
        if lvPendNone.SelCount > 0 then
          aLvObj := lvPendNone
        else if lvPendOut.SelCount > 0 then
          aLvObj := lvPendOut
        else if lvPendIn.SelCount > 0 then
          aLvObj := lvPendIn;
      end;
    end;
    // we must have a source object
    if not assigned(aLvObj) then
      exit;
    // Setup the progress bar
    if aLvObj is TORTreeView then
      ProgressStatusBar.ResetProgressBar(TORTreeView(aLvObj).SelectionCount * 3)
    else
      ProgressStatusBar.ResetProgressBar(TListView(aLvObj).SelCount * 3);
    // Perform the move
    if Source is TORTreeView then
    begin
      // we are moving a patient between new providers
      MoveItemsBetweenList(tvNewAssignments, aLvObj, nodeNewProvider);
      // Clean up items from the source
      RemoveItemsFromList(tvNewAssignments);
    end
    else
    begin
      // we are moving a patient(s) from old to new provider
      MoveItemsBetweenList(tvNewAssignments, aLvObj, nodeNewProvider);
      // Clean up items from the source
      RemoveItemsFromList(aLvObj);
    end;
    // Hide the progressbae
    ProgressStatusBar.HideProgress;
    ScrollBox1Resize(self);
  finally
    // Retrun the screen cursor
    screen.Cursor := rtnCursor;
  end;
end;

function TFormReassignOrders.ConstainsObj(X: TListView; anArray: tDropLocations): Boolean;
  var
    y : TListView;
    lComparer: IEqualityComparer<TListView>;
  begin
    lComparer := TEqualityComparer<TListView>.Default;
    for y in anArray do
    begin
      if lComparer.Equals(x, y) then
        Exit(True);
    end;
    Exit(False);
  end;

// CB
procedure TFormReassignOrders.tvNewAssignmentsDragOver(Sender, Source: TObject;
  X, Y: Integer; State: TDragState; var Accept: Boolean);
// *****************************************************************************
// approve a drop operation if the mouse is over a valid node of the tree list
// *****************************************************************************
begin
  // decline the drop if it is not over a valid tree node
  Accept := false;
  if tvNewAssignments.GetNodeAt(X, Y) = nil then
    exit
  else if Source is TListView then
    // accept the drop if the source is current patients list box
    Accept := true
  else if Source = tvNewAssignments then
  begin
    // accept the drop if source is tree list and node is a patient (not a provider)
    if (tvNewAssignments.Selected <> nil) and
      (tvNewAssignments.Selected.level > 0) then
      Accept := true;
  end;
end;

procedure TFormReassignOrders.tvNewAssignmentsStartDrag(Sender: TObject;
  var DragObject: TDragObject);
 procedure AddToDropList(XferRecord: tTransferToRec);
  var
   aOrdRec: tOrderRecord;
  begin
     aOrdRec := fOrderEngine.OrderByIEN(XferRecord.OrderIEN);
            case aOrdRec.OrderXfer of
              txftPendOut:
              begin
                if not ConstainsObj(lvPendOut, fDropLocations) then
                begin
                  SetLength(fDropLocations, Length(fDropLocations) + 1);
                  fDropLocations[high(fDropLocations)] := lvPendOut;
                end;
              end;
              txftPendIn:
              begin
                if not ConstainsObj(lvPendIn, fDropLocations) then
                begin
                  SetLength(fDropLocations, Length(fDropLocations) + 1);
                  fDropLocations[high(fDropLocations)] := lvPendIn;
                end;
              end;
              else begin
                if not ConstainsObj(lvPendNone, fDropLocations) then
                begin
                  SetLength(fDropLocations, Length(fDropLocations) + 1);
                  fDropLocations[high(fDropLocations)] := lvPendNone;
                end;
              end;
            end;
  end;

  procedure DetemineOrders();
  var
    I, x: integer;
  begin
    for I := 0 to tvNewAssignments.SelectionCount - 1 do
      begin
        if length(fDropLocations) = 3 then
          exit;
        //If the parent is selcetd then we will get the children when we hit the parent
        if Assigned(tvNewAssignments.Selections[I].Parent) and tvNewAssignments.Selections[I].Parent.Selected then
        begin
            Continue;
        end else begin
          if not tvNewAssignments.Selections[I].hasChildren then
            AddToDropList(fTransferOrdersLst.TransferRec[tvNewAssignments.Selections[I]])
          else begin
            for X := tvNewAssignments.Selections[I].Count - 1 downto 0 do
              AddToDropList(fTransferOrdersLst.TransferRec[tvNewAssignments.Selections[I].Item[X]])
          end;
        end;
      end;
  end;

begin
  inherited;
  if fViewType = tvtOrder then
  begin
    SetLength(fDropLocations, 0);
    DetemineOrders;
  end;
end;

// CB
procedure TFormReassignOrders.TimeOutAction;
// *****************************************************************************
// no user activity for specified period - close the application
// *****************************************************************************
begin
  Close;
end;

// CB
function TFormReassignOrders.TimeOutCondition: Boolean;
// *****************************************************************************
// N/A - not needed, but must be handled
// *****************************************************************************
begin
  result := false;
end;

end.
