unit uProg;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, ShellAnimations, Vcl.ExtCtrls,
  Vcl.StdCtrls, Vcl.ComCtrls;

type
  TStopWatch = class
  private
    FOwner: TObject;
    fFrequency: TLargeInteger;
    fIsRunning: Boolean;
    fIsHighResolution: Boolean;
    fStartCount, fStopCount: TLargeInteger;
    fActive: Boolean;
    procedure SetTickStamp(var lInt: TLargeInteger);
    function GetElapsedTicks: TLargeInteger;
    function GetElapsedMilliseconds: TLargeInteger;
    Function GetElapsedNanoSeconds: TLargeInteger;
    function GetElapsed: string;
  public
    constructor Create(AOwner: TComponent; const IsActive: Boolean = false;
      const startOnCreate: Boolean = false);
    destructor Destroy; override;
    procedure Start;
    procedure Stop;
    property IsHighResolution: Boolean read fIsHighResolution;
    property ElapsedTicks: TLargeInteger read GetElapsedTicks;
    property ElapsedMilliseconds: TLargeInteger read GetElapsedMilliseconds;
    property ElapsedNanoSeconds: TLargeInteger read GetElapsedNanoSeconds;
    property Elapsed: string read GetElapsed;
    property IsRunning: Boolean read fIsRunning;
  end;


  TfrmProg = class(TForm)
    pnlBottom: TPanel;
    btnOK: TButton;
    Animate1: TAnimate;
    ProgressBar1: TProgressBar;
    StaticText1: TStaticText;
    Timer1: TTimer;
    StaticText2: TStaticText;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    fStopTimer: TStopWatch;
        fCrntLoop: Integer;
    fMaxLoop: Integer;
    procedure SetCrntLoop(const Value: Integer);
  public
    { Public declarations }
    Property CurrentLoop: Integer read fCrntLoop write SetCrntLoop;
    Property MaxLoop: Integer read fMaxLoop write fMaxLoop;
  end;

var
  frmProg: TfrmProg;

implementation

uses
  system.DateUtils, System.Math;

{$R *.dfm}
{$REGION 'TStopWatch'}

Const
  NSecsPerSec = 1000000000;

constructor TStopWatch.Create(AOwner: TComponent;
const IsActive: Boolean = false; const startOnCreate: Boolean = false);
begin
  inherited Create();
  FOwner := AOwner;
  fIsRunning := false;
  fActive := IsActive;
  fIsHighResolution := QueryPerformanceFrequency(fFrequency);
  if NOT fIsHighResolution then
    fFrequency := MSecsPerSec;

  if startOnCreate then
    Start;
end;

destructor TStopWatch.Destroy;
begin
  Stop;
  inherited Destroy;
end;

function TStopWatch.GetElapsedTicks: TLargeInteger;
begin
  Result := fStopCount - fStartCount;
end;

procedure TStopWatch.SetTickStamp(var lInt: TLargeInteger);
begin
  if fIsHighResolution then
    QueryPerformanceCounter(lInt)
  else
    lInt := MilliSecondOf(Now);
end;

function TStopWatch.GetElapsed: string;
begin
  Result := FloatToStr(ElapsedMilliseconds / 1000) + ' Sec / ' +
    FloatToStr(ElapsedMilliseconds) + ' Ms / ' +
    FloatToStr(ElapsedNanoSeconds) + ' Ns';
end;

function TStopWatch.GetElapsedMilliseconds: TLargeInteger;
var
  Crnt: TLargeInteger;
begin
  if fIsRunning then
  begin
    SetTickStamp(Crnt);
    Result := (MSecsPerSec * (Crnt - fStartCount)) div fFrequency;
  end
  else
    Result := (MSecsPerSec * (fStopCount - fStartCount)) div fFrequency;
end;

function TStopWatch.GetElapsedNanoSeconds: TLargeInteger;
begin
  Result := (NSecsPerSec * (fStopCount - fStartCount)) div fFrequency;
end;

procedure TStopWatch.Start;
begin
  if fActive then
  begin
    SetTickStamp(fStartCount);
    fIsRunning := true;
  end;
end;

procedure TStopWatch.Stop;
begin
  if fActive then
  begin
    SetTickStamp(fStopCount);
    fIsRunning := false;
  end;
end;
{$ENDREGION}



procedure TfrmProg.FormCreate(Sender: TObject);
begin
   fStopTimer := TStopWatch.Create(Application, true);
   SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NoMove or SWP_NoSize);
end;

procedure TfrmProg.FormDestroy(Sender: TObject);
begin
  FreeandNil(fStopTimer);
end;

procedure TfrmProg.FormHide(Sender: TObject);
begin
 Timer1.Enabled := false;
 fStopTimer.Stop;
end;

procedure TfrmProg.FormShow(Sender: TObject);
begin
 if not Timer1.Enabled then
    Timer1.Enabled := true;
end;

procedure TfrmProg.SetCrntLoop(const Value: Integer);
begin
  fCrntLoop := Value;
end;

procedure TfrmProg.Timer1Timer(Sender: TObject);
var
  atmp: Double;
begin
  if not fStopTimer.IsRunning then
    fStopTimer.Start;
  if (fCrntLoop) > 0 then
  begin
    atmp := ((fStopTimer.ElapsedMilliseconds / 1000) / fCrntLoop) *
      (fMaxLoop - fCrntLoop);

    StaticText1.Caption := 'Estimated ' + FormatDateTime('hh:nn:ss',
      System.Math.ceil(atmp) / SecsPerDay) + ' left ';
    { - ' + IntToStr(CurrentLoop)
      + ' of ' + IntToStr(MaxLoops); }

    // sleep(0);
    Application.ProcessMessages;
  end;
  if fCrntLoop >= fMaxLoop then
  begin
    fStopTimer.Stop;
    Timer1.Enabled := false;
  end;
end;

end.
