unit uSizeEngine;

interface

uses SysUtils, Classes, Controls;

type
  TSizeHolder = class(TObject)
  private
    FSizeList: TStringList;
    function GetObjectName(const aObj: TWinControl): String;
    Function SizeToStr(const aObj: TWinControl): String;
  public
    constructor Create;
    destructor Destroy; override;
    procedure LoadSize(const aObj: TWinControl);
    procedure SetSize(const aObj: TWinControl);
    procedure LoadAllSizes;
    Function SaveAllSizes: String;
  end;

  SettingSizeArray = array of integer;

Const
  U = '^';

var
  SizeHolder: TSizeHolder;

implementation

uses
  ornet, orfn, forms, comctrls;

constructor TSizeHolder.Create;
begin
  inherited;
  FSizeList := TStringList.Create;
end;

destructor TSizeHolder.Destroy;
begin
  FSizeList.Free;
  inherited;
end;

procedure TSizeHolder.LoadSize(const aObj: TWinControl);
var
  ObjName, ObjSize: String;
  I: integer;
begin
  ObjName := GetObjectName(aObj);
  if FSizeList.IndexOfName(ObjName) > -1 then
  begin
    ObjSize := FSizeList.Values[ObjName];
    if (ObjSize = '0,0,0,0') and (aObj is TForm) then
      TForm(aObj).WindowState := wsMaximized
    else
    begin
      if (aObj.Align = alNone) or (aObj.Align = alCustom) then
      begin
        aObj.Left := HigherOf(StrToIntDef(Piece(ObjSize, ',', 1), aObj.Left), 0);
        aObj.Top := HigherOf(StrToIntDef(Piece(ObjSize, ',', 2), aObj.Top), 0);
      end;

   {   if Assigned(aObj.Parent) then
      begin
        aObj.Width := LowerOf(StrToIntDef(Piece(ObjSize, ',', 3), aObj.Width),
          aObj.Parent.Width - aObj.Left);
        aObj.Height := LowerOf(StrToIntDef(Piece(ObjSize, ',', 4), aObj.Height),
          aObj.Parent.Height - aObj.Top);
      end
      else
      begin}
        aObj.Width := StrToIntDef(Piece(ObjSize, ',', 3), aObj.Width);
        aObj.Height := StrToIntDef(Piece(ObjSize, ',', 4), aObj.Height);
     // end;

      //If listview then try to get the columns
      if (aObj is TListView) then
      begin
        for I := 0 to TListView(aObj).Columns.Count -1 do
        begin
           if FSizeList.IndexOfName(ObjName+'_COL'+IntToStr(I)) > -1 then
           begin
            ObjSize := FSizeList.Values[ObjName+'_COL'+IntToStr(I)];
            TListView(aObj).Column[i].Width := StrToIntDef(Piece(ObjSize, ',', 3), TListView(aObj).Column[i].Width);
           end;
        end;
      end;


      if (aObj is TForm) then
        TForm(aObj).MakeFullyVisible;
    end;
  end;
end;

procedure TSizeHolder.SetSize(const aObj: TWinControl);
var
  ObjName, ObjSize: String;
  I: Integer;
begin
  ObjName := GetObjectName(aObj);
  ObjSize := SizeToStr(aObj);
  if FSizeList.IndexOfName(ObjName) > -1 then
    FSizeList.Values[ObjName] := ObjSize
  else
    FSizeList.Add(ObjName + '=' + ObjSize);

  //If listview then try to get the columns
  if (aObj is TListView) then
  begin
    for I := 0 to TListView(aObj).Columns.Count -1 do
    begin
      if FSizeList.IndexOfName(ObjName+'_COL'+IntToStr(I)) > -1 then
       FSizeList.Values[ObjName+'_COL'+IntToStr(I)] := '0,0,'+IntToStr(TListView(aObj).Column[i].Width)+',0'
      else
        FSizeList.Add(ObjName+'_COL'+IntToStr(I)+'='+'0,0,'+IntToStr(TListView(aObj).Column[i].Width)+',0');
    end;
  end;
end;

procedure TSizeHolder.LoadAllSizes;
var
  tmpStrLst: TStringList;
  tmpStr: String;
begin
  tmpStrLst := TStringList.Create;
  try
    FSizeList.Clear;
    CallVistA('ORQ3 LOADALL', [nil], tmpStrLst);
    for tmpStr in tmpStrLst do
      FSizeList.Add(Piece(tmpStr, U, 1) + '=' + Piece(tmpStr, U, 2));
  finally
    tmpStrLst.Free;
  end;
end;

function TSizeHolder.SaveAllSizes: String;
var
  tmpStrLst: TStringList;
  tmpStr: String;
begin
 tmpStrLst := TStringList.Create;
  try
   for tmpStr in FSizeList do
      tmpStrLst.Add(StringReplace(tmpStr, '=', '^', [rfReplaceAll]));
    CallVistA('ORQ3 SAVEALL', [tmpStrLst], Result);
  finally
    tmpStrLst.Free;
  end;
end;

function TSizeHolder.GetObjectName(const aObj: TWinControl): String;
begin
  Result := aObj.Name;
  if not(aObj is TForm) and (Assigned(aObj.Owner)) then
    Result := aObj.Owner.Name + '.' + Result;
end;

Function TSizeHolder.SizeToStr(const aObj: TWinControl): String;
begin
  // x := GetObjectName(aObj);

  with aObj do
    Result := { x + U + } IntToStr(Left) + ',' + IntToStr(Top) + ',' +
      IntToStr(Width) + ',' + IntToStr(Height);
  if (aObj is TForm) and (TForm(aObj).WindowState = wsMaximized) then
    Result := { x + U + } '0,0,0,0';
end;

end.
